@@ -3,21 +3,21 @@
 %%% Author  : Alexey Shchepin <alexey@sevcom.net>
 %%% Purpose : Cyrus SASL-like library
 %%% Created :  8 Mar 2003 by Alexey Shchepin <alexey@sevcom.net>
-%%% Id      : $Id: cyrsasl.erl 307 2005-04-17 18:08:34Z tmallard $
+%%% Id      : $Id: cyrsasl.erl 374 2005-07-13 03:24:13Z alexey $
 %%%----------------------------------------------------------------------
 
 -module(cyrsasl).
 -author('alexey@sevcom.net').
--vsn('$Revision$ ').
+-vsn('$Revision: 374 $ ').
 
 -export([start/0,
-	 register_mechanism/2,
-	 listmech/0,
+	 register_mechanism/3,
+	 listmech/1,
 	 server_new/6,
 	 server_start/3,
 	 server_step/2]).
 
--record(sasl_mechanism, {mechanism, module}).
+-record(sasl_mechanism, {mechanism, module, require_plain_password}).
 -record(sasl_state, {service, myname, realm,
 		     get_password, check_password,
 		     mech_mod, mech_state}).
@@ -38,9 +38,11 @@ start() ->
     cyrsasl_digest:start([]),
     ok.
 
-register_mechanism(Mechanism, Module) ->
-    ets:insert(sasl_mechanism, #sasl_mechanism{mechanism = Mechanism,
-					       module = Module}).
+register_mechanism(Mechanism, Module, RequirePlainPassword) ->
+    ets:insert(sasl_mechanism,
+	       #sasl_mechanism{mechanism = Mechanism,
+			       module = Module,
+			       require_plain_password = RequirePlainPassword}).
 
 % TODO: use callbacks
 -include("ejabberd.hrl").
@@ -77,9 +79,19 @@ check_credentials(State, Props) ->
 	    ok
     end.
 
-listmech() ->
+listmech(Host) ->
+    RequirePlainPassword = ejabberd_auth:plain_password_required(Host),
     ets:select(sasl_mechanism,
-	       [{#sasl_mechanism{mechanism = '$1', _ = '_'}, [], ['$1']}]).
+	       [{#sasl_mechanism{mechanism = '$1',
+				 require_plain_password = '$2',
+				 _ = '_'},
+		 if
+		     RequirePlainPassword ->
+			 [{'==', '$2', false}];
+		     true ->
+			 []
+		 end,
+		 ['$1']}]).
 
 
 server_new(Service, ServerFQDN, UserRealm, SecFlags,
