@@ -3,12 +3,12 @@
 %%% Author  : Alexey Shchepin <alexey@sevcom.net>
 %%% Purpose : DIGEST-MD5 SASL mechanism
 %%% Created : 11 Mar 2003 by Alexey Shchepin <alexey@sevcom.net>
-%%% Id      : $Id: cyrsasl_digest.erl 307 2005-04-17 18:08:34Z tmallard $
+%%% Id      : $Id: cyrsasl_digest.erl 405 2005-08-10 23:29:16Z alexey $
 %%%----------------------------------------------------------------------
 
 -module(cyrsasl_digest).
 -author('alexey@sevcom.net').
--vsn('$Revision$ ').
+-vsn('$Revision: 405 $ ').
 
 -export([start/1,
 	 stop/0,
@@ -20,13 +20,7 @@
 -record(state, {step, nonce, username, authzid, get_password}).
 
 start(_Opts) ->
-    case ejabberd_auth:plain_password_required() of
-	true ->
-	    ok;
-	false ->
-	    cyrsasl:register_mechanism("DIGEST-MD5", ?MODULE),
-	    ok
-    end.
+    cyrsasl:register_mechanism("DIGEST-MD5", ?MODULE, true).
 
 stop() ->
     ok.
@@ -83,6 +77,10 @@ parse(S) ->
 
 parse1([$= | Cs], S, Ts) ->
     parse2(Cs, lists:reverse(S), "", Ts);
+parse1([$, | Cs], [], Ts) ->
+    parse1(Cs, [], Ts);
+parse1([$\s | Cs], [], Ts) ->
+    parse1(Cs, [], Ts);
 parse1([C | Cs], S, Ts) ->
     parse1(Cs, [C | S], Ts);
 parse1([], [], T) ->
@@ -106,6 +104,8 @@ parse3([], _, _, _) ->
 
 parse4([$, | Cs], Key, Val, Ts) ->
     parse1(Cs, "", [{Key, lists:reverse(Val)} | Ts]);
+parse4([$\s | Cs], Key, Val, Ts) ->
+    parse4(Cs, Key, Val, Ts);
 parse4([C | Cs], Key, Val, Ts) ->
     parse4(Cs, Key, [C | Val], Ts);
 parse4([], Key, Val, Ts) ->
