@@ -3,12 +3,12 @@
 %%% Author  : Alexey Shchepin <alexey@sevcom.net>
 %%% Purpose : PLAIN SASL mechanism
 %%% Created :  8 Mar 2003 by Alexey Shchepin <alexey@sevcom.net>
-%%% Id      : $Id: cyrsasl_plain.erl 307 2005-04-17 18:08:34Z tmallard $
+%%% Id      : $Id: cyrsasl_plain.erl 374 2005-07-13 03:24:13Z alexey $
 %%%----------------------------------------------------------------------
 
 -module(cyrsasl_plain).
 -author('alexey@sevcom.net').
--vsn('$Revision$ ').
+-vsn('$Revision: 374 $ ').
 
 -export([start/1, stop/0, mech_new/2, mech_step/2, parse/1]).
 
@@ -17,7 +17,7 @@
 -record(state, {check_password}).
 
 start(_Opts) ->
-    cyrsasl:register_mechanism("PLAIN", ?MODULE),
+    cyrsasl:register_mechanism("PLAIN", ?MODULE, false),
     ok.
 
 stop() ->
