@@ -3,19 +3,17 @@
 %%% Author  : Alexey Shchepin <alexey@sevcom.net>
 %%% Purpose : 
 %%% Created : 31 Jan 2003 by Alexey Shchepin <alexey@sevcom.net>
-%%% Id      : $Id: ejabberd_app.erl 223 2004-04-15 19:55:38Z aleksey $
+%%% Id      : $Id: ejabberd_app.erl 427 2005-10-25 02:19:01Z alexey $
 %%%----------------------------------------------------------------------
 
 -module(ejabberd_app).
 -author('alexey@sevcom.net').
--vsn('$Revision$ ').
+-vsn('$Revision: 427 $ ').
 
 -behaviour(application).
 
 -export([start/2, stop/1, init/0]).
 
--export([dump_ports/0]).
-
 -include("ejabberd.hrl").
 
 start(normal, _Args) ->
@@ -28,13 +26,13 @@ start(normal, _Args) ->
     acl:start(),
     gen_mod:start(),
     ejabberd_config:start(),
+    Sup = ejabberd_sup:start_link(),
     ejabberd_auth:start(),
     cyrsasl:start(),
     % Profiling
     %eprof:start(),
     %eprof:profile([self()]),
     %fprof:trace(start, "/tmp/fprof"),
-    Sup = ejabberd_sup:start_link(),
     start(),
     load_modules(),
     Sup;
@@ -64,8 +62,11 @@ init() ->
 		end
 	end,
     error_logger:add_report_handler(ejabberd_logger_h, LogPath),
-    %timer:apply_interval(3600000, ?MODULE, dump_ports, []),
-    ok = erl_ddll:load_driver(ejabberd:get_so_path(), expat_erl),
+    erl_ddll:load_driver(ejabberd:get_so_path(), tls_drv),
+    case erl_ddll:load_driver(ejabberd:get_so_path(), expat_erl) of
+	ok -> ok;
+	{error, already_loaded} -> ok
+    end,
     Port = open_port({spawn, expat_erl}, [binary]),
     loop(Port).
 
@@ -87,17 +88,16 @@ db_init() ->
     mnesia:wait_for_tables(mnesia:system_info(local_tables), infinity).
 
 load_modules() ->
-    case ejabberd_config:get_local_option(modules) of
-	undefined ->
-	    ok;
-	Modules ->
-	    lists:foreach(fun({Module, Args}) ->
-				  gen_mod:start_module(Module, Args)
-			  end, Modules)
-    end.
-
-
-dump_ports() ->
-    ?INFO_MSG("ports:~n ~p",
-	      [lists:map(fun(P) -> erlang:port_info(P) end, erlang:ports())]).
+    lists:foreach(
+      fun(Host) ->
+	      case ejabberd_config:get_local_option({modules, Host}) of
+		  undefined ->
+		      ok;
+		  Modules ->
+		      lists:foreach(
+			fun({Module, Args}) ->
+				gen_mod:start_module(Host, Module, Args)
+			end, Modules)
+	      end
+      end, ?MYHOSTS).
 
