@@ -3,12 +3,12 @@
 %%% Author  : Alexey Shchepin <alexey@sevcom.net>
 %%% Purpose : Authentification
 %%% Created : 23 Nov 2002 by Alexey Shchepin <alexey@sevcom.net>
-%%% Id      : $Id: ejabberd_auth.erl 323 2005-04-19 23:10:22Z alexey $
+%%% Id      : $Id: ejabberd_auth.erl 374 2005-07-13 03:24:13Z alexey $
 %%%----------------------------------------------------------------------
 
 -module(ejabberd_auth).
 -author('alexey@sevcom.net').
--vsn('$Revision$ ').
+-vsn('$Revision: 374 $ ').
 
 %% External exports
 -export([start/0,
@@ -23,7 +23,7 @@
 	 is_user_exists/2,
 	 remove_user/2,
 	 remove_user/3,
-	 plain_password_required/0
+	 plain_password_required/1
 	]).
 
 -include("ejabberd.hrl").
@@ -32,55 +32,58 @@
 %%% API
 %%%----------------------------------------------------------------------
 start() ->
-    (auth_module()):start().
+    lists:foreach(fun(Host) ->
+			  (auth_module(Host)):start(Host)
+		  end, ?MYHOSTS).
 
-plain_password_required() ->
-    (auth_module()):plain_password_required().
+plain_password_required(Server) ->
+    (auth_module(Server)):plain_password_required().
 
 check_password(User, Server, Password) ->
-    (auth_module()):check_password(User, Server, Password).
+    (auth_module(Server)):check_password(User, Server, Password).
 
 check_password(User, Server, Password, StreamID, Digest) ->
-    (auth_module()):check_password(User, Server, Password, StreamID, Digest).
+    (auth_module(Server)):check_password(User, Server, Password, StreamID, Digest).
 
 set_password(User, Server, Password) ->
-    (auth_module()):set_password(User, Server, Password).
+    (auth_module(Server)):set_password(User, Server, Password).
 
 try_register(User, Server, Password) ->
     case lists:member(jlib:nameprep(Server), ?MYHOSTS) of
 	true ->
-	    (auth_module()):try_register(User, Server, Password);
+	    (auth_module(Server)):try_register(User, Server, Password);
 	false ->
 	    {error, not_allowed}
     end.
 
 dirty_get_registered_users() ->
-    (auth_module()):dirty_get_registered_users().
+    (auth_module(?MYNAME)):dirty_get_registered_users().
 
 get_vh_registered_users(Server) ->
-    (auth_module()):get_vh_registered_users(Server).
+    (auth_module(Server)):get_vh_registered_users(Server).
 
 get_password(User, Server) ->
-    (auth_module()):get_password(User, Server).
+    (auth_module(Server)):get_password(User, Server).
 
 get_password_s(User, Server) ->
-    (auth_module()):get_password_s(User, Server).
+    (auth_module(Server)):get_password_s(User, Server).
 
 is_user_exists(User, Server) ->
-    (auth_module()):is_user_exists(User, Server).
+    (auth_module(Server)):is_user_exists(User, Server).
 
 remove_user(User, Server) ->
-    (auth_module()):remove_user(User, Server).
+    (auth_module(Server)):remove_user(User, Server).
 
 remove_user(User, Server, Password) ->
-    (auth_module()):remove_user(User, Server, Password).
+    (auth_module(Server)):remove_user(User, Server, Password).
 
 %%%----------------------------------------------------------------------
 %%% Internal functions
 %%%----------------------------------------------------------------------
 
-auth_module() ->
-    case ejabberd_config:get_local_option(auth_method) of
+auth_module(Server) ->
+    LServer = jlib:nameprep(Server),
+    case ejabberd_config:get_local_option({auth_method, LServer}) of
 	external ->
 	    ejabberd_auth_external;
 	ldap ->
