@@ -3,15 +3,15 @@
 %%% Author  : Alexey Shchepin <alexey@sevcom.net>
 %%% Purpose : Authentification via mnesia
 %%% Created : 12 Dec 2004 by Alexey Shchepin <alexey@sevcom.net>
-%%% Id      : $Id: ejabberd_auth_internal.erl 307 2005-04-17 18:08:34Z tmallard $
+%%% Id      : $Id: ejabberd_auth_internal.erl 374 2005-07-13 03:24:13Z alexey $
 %%%----------------------------------------------------------------------
 
 -module(ejabberd_auth_internal).
 -author('alexey@sevcom.net').
--vsn('$Revision$ ').
+-vsn('$Revision: 374 $ ').
 
 %% External exports
--export([start/0,
+-export([start/1,
 	 set_password/3,
 	 check_password/3,
 	 check_password/5,
@@ -33,7 +33,7 @@
 %%%----------------------------------------------------------------------
 %%% API
 %%%----------------------------------------------------------------------
-start() ->
+start(_Host) ->
     mnesia:create_table(passwd, [{disc_copies, [node()]},
 				 {attributes, record_info(fields, passwd)}]),
     update_table(),
@@ -165,7 +165,7 @@ remove_user(User, Server) ->
 		mnesia:delete({passwd, US})
         end,
     mnesia:transaction(F),
-    ejabberd_hooks:run(remove_user, [User, Server]).
+    ejabberd_hooks:run(remove_user, LServer, [User, Server]).
 
 remove_user(User, Server, Password) ->
     LUser = jlib:nodeprep(User),
@@ -184,7 +184,7 @@ remove_user(User, Server, Password) ->
         end,
     case mnesia:transaction(F) of
 	{atomic, ok} ->
-	    ejabberd_hooks:run(remove_user, [User, Server]),
+	    ejabberd_hooks:run(remove_user, LServer, [User, Server]),
 	    ok;
 	{atomic, Res} ->
 	    Res;
