@@ -3,12 +3,12 @@
 %%% Author  : Alexey Shchepin <alexey@sevcom.net>
 %%% Purpose : Serve C2S connection
 %%% Created : 16 Nov 2002 by Alexey Shchepin <alexey@sevcom.net>
-%%% Id      : $Id: ejabberd_c2s.erl 345 2005-05-09 19:12:41Z alexey $
+%%% Id      : $Id: ejabberd_c2s.erl 440 2005-11-22 18:00:56Z alexey $
 %%%----------------------------------------------------------------------
 
 -module(ejabberd_c2s).
 -author('alexey@sevcom.net').
--vsn('$Revision: 345 $ ').
+-vsn('$Revision: 440 $ ').
 
 -behaviour(gen_fsm).
 
@@ -49,7 +49,7 @@
 		tls_required = false,
 		tls_enabled = false,
 		tls_options = [],
-		authentificated = false,
+		authenticated = false,
 		jid,
 		user = "", server = ?MYNAME, resource = "",
 		pres_t = ?SETS:new(),
@@ -178,7 +178,7 @@ wait_for_stream({xmlstreamstart, _Name, Attrs}, StateData) ->
 						    " version='1.0'",
 						    DefaultLang]),
 			    send_text(StateData, Header),
-			    case StateData#state.authentificated of
+			    case StateData#state.authenticated of
 				false ->
 				    SASLState =
 					cyrsasl:server_new(
@@ -195,7 +195,7 @@ wait_for_stream({xmlstreamstart, _Name, Attrs}, StateData) ->
 					      fun(S) ->
 						      {xmlelement, "mechanism", [],
 						       [{xmlcdata, S}]}
-					      end, cyrsasl:listmech()),
+					      end, cyrsasl:listmech(Server)),
 				    TLS = StateData#state.tls,
 				    TLSEnabled = StateData#state.tls_enabled,
 				    TLSRequired = StateData#state.tls_required,
@@ -223,10 +223,11 @@ wait_for_stream({xmlstreamstart, _Name, Attrs}, StateData) ->
 						  TLSFeature ++
 						  [{xmlelement, "mechanisms",
 						    [{"xmlns", ?NS_SASL}],
-						    Mechs},
-						   {xmlelement, "register",
-						    [{"xmlns", ?NS_FEATURE_IQREGISTER}],
-						    []}]}),
+						    Mechs}] ++
+						   ejabberd_hooks:run_fold(
+						     c2s_stream_features,
+						     Server,
+						     [], [])}),
 				    {next_state, wait_for_feature_request,
 				     StateData#state{server = Server,
 						     sasl_state = SASLState,
@@ -312,7 +313,8 @@ wait_for_auth({xmlstreamelement, El}, StateData) ->
 	    {next_state, wait_for_auth, StateData};
 	{auth, _ID, get, {U, _, _, _}} ->
 	    {xmlelement, Name, Attrs, _Els} = jlib:make_result_iq_reply(El),
-	    Res = case ejabberd_auth:plain_password_required() of
+	    Res = case ejabberd_auth:plain_password_required(
+			 StateData#state.server) of
 		      false ->
 			  {xmlelement, Name, Attrs,
 			   [{xmlelement, "query", [{"xmlns", ?NS_AUTH}],
@@ -340,7 +342,8 @@ wait_for_auth({xmlstreamelement, El}, StateData) ->
 	{auth, _ID, set, {U, P, D, R}} ->
 	    JID = jlib:make_jid(U, StateData#state.server, R),
 	    case (JID /= error) andalso
-		(acl:match_rule(StateData#state.access, JID) == allow) of
+		(acl:match_rule(StateData#state.server,
+				StateData#state.access, JID) == allow) of
 		true ->
 		    case ejabberd_auth:check_password(
 			   U, StateData#state.server, P,
@@ -358,6 +361,7 @@ wait_for_auth({xmlstreamelement, El}, StateData) ->
 			    change_shaper(StateData, JID),
 			    {Fs, Ts} = ejabberd_hooks:run_fold(
 					 roster_get_subscription_lists,
+					 StateData#state.server,
 					 {[], []},
 					 [U, StateData#state.server]),
 			    LJID = jlib:jid_tolower(
@@ -408,21 +412,8 @@ wait_for_auth({xmlstreamelement, El}, StateData) ->
 		    end
 	    end;
 	_ ->
-	    case jlib:iq_query_info(El) of
-		#iq{xmlns = ?NS_REGISTER} = IQ ->
-		    ResIQ = mod_register:process_iq(
-			      {"", "", ""},
-			      jlib:make_jid("", StateData#state.server, ""),
-			      IQ),
-		    Res1 = jlib:replace_from_to({"", StateData#state.server, ""},
-						{"", "", ""},
-						jlib:iq_to_xml(ResIQ)),
-		    Res = jlib:remove_attr("to", Res1),
-		    send_element(StateData, Res),
-		    {next_state, wait_for_auth, StateData};
-		_ ->
-		    {next_state, wait_for_auth, StateData}
-	    end
+	    process_unauthenticated_stanza(StateData, El),
+	    {next_state, wait_for_auth, StateData}
     end;
 
 wait_for_auth({xmlstreamend, _Name}, StateData) ->
@@ -460,7 +451,7 @@ wait_for_feature_request({xmlstreamelement, El}, StateData) ->
 			      [StateData#state.socket, U]),
 		    {next_state, wait_for_stream,
 		     StateData#state{streamid = new_id(),
-				     authentificated = true,
+				     authenticated = true,
 				     user = U
 				    }};
 		{continue, ServerOut, NewSASLState} ->
@@ -482,7 +473,16 @@ wait_for_feature_request({xmlstreamelement, El}, StateData) ->
 				   TLSEnabled == false,
 				   SockMod == gen_tcp ->
 	    Socket = StateData#state.socket,
-	    TLSOpts = StateData#state.tls_options,
+	    TLSOpts = case ejabberd_config:get_local_option(
+			     {domain_certfile, StateData#state.server}) of
+			  undefined ->
+			      StateData#state.tls_options;
+			  CertFile ->
+			      [{certfile, CertFile} |
+			       lists:keydelete(
+				 certfile, 1, StateData#state.tls_options)]
+		      end,
+	    io:format("O: ~p~n", [TLSOpts]),
 	    {ok, TLSSocket} = tls:tcp_to_tls(Socket, TLSOpts),
 	    ejabberd_receiver:starttls(StateData#state.receiver, TLSSocket),
 	    send_element(StateData,
@@ -503,21 +503,8 @@ wait_for_feature_request({xmlstreamelement, El}, StateData) ->
 					 ?STREAM_TRAILER),
 		    {stop, normal, StateData};
 		true ->
-		    case jlib:iq_query_info(El) of
-			#iq{xmlns = ?NS_REGISTER} = IQ ->
-			    ResIQ = mod_register:process_iq(
-				      {"", "", ""},
-				      jlib:make_jid("", StateData#state.server, ""),
-				      IQ),
-			    Res1 = jlib:replace_from_to({"", StateData#state.server, ""},
-							{"", "", ""},
-							jlib:iq_to_xml(ResIQ)),
-			    Res = jlib:remove_attr("to", Res1),
-			    send_element(StateData, Res),
-			    {next_state, wait_for_feature_request, StateData};
-			_ ->
-			    {next_state, wait_for_feature_request, StateData}
-		    end
+		    process_unauthenticated_stanza(StateData, El),
+		    {next_state, wait_for_feature_request, StateData}
 	    end
     end;
 
@@ -550,7 +537,7 @@ wait_for_sasl_response({xmlstreamelement, El}, StateData) ->
 			      [StateData#state.socket, U]),
 		    {next_state, wait_for_stream,
 		     StateData#state{streamid = new_id(),
-				     authentificated = true,
+				     authenticated = true,
 				     user = U
 				    }};
 		{continue, ServerOut, NewSASLState} ->
@@ -569,21 +556,8 @@ wait_for_sasl_response({xmlstreamelement, El}, StateData) ->
 		    {next_state, wait_for_feature_request, StateData}
 	    end;
 	_ ->
-	    case jlib:iq_query_info(El) of
-		#iq{xmlns = ?NS_REGISTER} = IQ ->
-		    ResIQ = mod_register:process_iq(
-			      {"", "", ""},
-			      jlib:make_jid("", StateData#state.server, ""),
-			      IQ),
-		    Res1 = jlib:replace_from_to({"", StateData#state.server, ""},
-						{"", "", ""},
-						jlib:iq_to_xml(ResIQ)),
-		    Res = jlib:remove_attr("to", Res1),
-		    send_element(StateData, Res),
-		    {next_state, wait_for_feature_request, StateData};
-		_ ->
-		    {next_state, wait_for_feature_request, StateData}
-	    end
+	    process_unauthenticated_stanza(StateData, El),
+	    {next_state, wait_for_feature_request, StateData}
     end;
 
 wait_for_sasl_response({xmlstreamend, _Name}, StateData) ->
@@ -651,7 +625,8 @@ wait_for_session({xmlstreamelement, El}, StateData) ->
 	    U = StateData#state.user,
 	    R = StateData#state.resource,
 	    JID = StateData#state.jid,
-	    case acl:match_rule(StateData#state.access, JID) of
+	    case acl:match_rule(StateData#state.server,
+				StateData#state.access, JID) of
 		allow ->
 		    ?INFO_MSG("(~w) Opened session for ~s",
 			      [StateData#state.socket,
@@ -663,6 +638,7 @@ wait_for_session({xmlstreamelement, El}, StateData) ->
 		    change_shaper(StateData, JID),
 		    {Fs, Ts} = ejabberd_hooks:run_fold(
 				 roster_get_subscription_lists,
+				 StateData#state.server,
 				 {[], []},
 				 [U, StateData#state.server]),
 		    LJID = jlib:jid_tolower(jlib:jid_remove_resource(JID)),
@@ -764,6 +740,7 @@ session_established({xmlstreamelement, El}, StateData) ->
 				    _ ->
 					ejabberd_hooks:run(
 					  user_send_packet,
+					  Server,
 					  [FromJID, ToJID, NewEl]),
 					ejabberd_router:route(
 					  FromJID, ToJID, NewEl),
@@ -772,6 +749,7 @@ session_established({xmlstreamelement, El}, StateData) ->
 			end;
 		    "message" ->
 			ejabberd_hooks:run(user_send_packet,
+					   Server,
 					   [FromJID, ToJID, NewEl]),
 			ejabberd_router:route(FromJID, ToJID, NewEl),
 			StateData;
@@ -854,9 +832,12 @@ handle_info({send_text, Text}, StateName, StateData) ->
     send_text(StateData, Text),
     {next_state, StateName, StateData};
 handle_info(replaced, _StateName, StateData) ->
-    % TODO
-    %send_text(StateData#state.sender, Text),
-    {stop, normal, StateData#state{authentificated = replaced}};
+    Lang = StateData#state.lang,
+    send_text(StateData,
+	      xml:element_to_string(
+		?SERRT_CONFLICT(Lang, "Replaced by new connection"))
+	      ++ ?STREAM_TRAILER),
+    {stop, normal, StateData#state{authenticated = replaced}};
 handle_info({route, From, To, Packet}, StateName, StateData) ->
     {xmlelement, Name, Attrs, Els} = Packet,
     {Pass, NewAttrs, NewState} =
@@ -926,10 +907,16 @@ handle_info({route, From, To, Packet}, StateName, StateData) ->
 		IQ = jlib:iq_query_info(Packet),
 		case IQ of
 		    #iq{xmlns = ?NS_VCARD} ->
-			ResIQ = mod_vcard:process_sm_iq(From, To, IQ),
-			ejabberd_router:route(To,
-					      From,
-					      jlib:iq_to_xml(ResIQ)),
+			Host = StateData#state.server,
+			case ets:lookup(sm_iqtable, {?NS_VCARD, Host}) of
+			    [{_, Module, Function, Opts}] ->
+				gen_iq_handler:handle(Host, Module, Function, Opts,
+						      From, To, IQ);
+			    [] ->
+				Err = jlib:make_error_reply(
+					Packet, ?ERR_FEATURE_NOT_IMPLEMENTED),
+				ejabberd_router:route(To, From, Err)
+			end,
 			{false, Attrs, StateData};
 %-ifdef(PRIVACY_SUPPORT).
 		    #iq{} ->
@@ -983,6 +970,7 @@ handle_info({route, From, To, Packet}, StateName, StateData) ->
 	    Text = xml:element_to_string(FixedPacket),
 	    send_text(StateData, Text),
 	    ejabberd_hooks:run(user_receive_packet,
+			       StateData#state.server,
 			       [StateData#state.jid, From, To, FixedPacket]),
 	    {next_state, StateName, NewState};
 	true ->
@@ -997,7 +985,7 @@ handle_info({route, From, To, Packet}, StateName, StateData) ->
 terminate(_Reason, StateName, StateData) ->
     case StateName of
 	session_established ->
-	    case StateData#state.authentificated of
+	    case StateData#state.authenticated of
 		replaced ->
 		    ?INFO_MSG("(~w) Replaced session for ~s",
 			      [StateData#state.socket,
@@ -1055,7 +1043,8 @@ terminate(_Reason, StateName, StateData) ->
 %%%----------------------------------------------------------------------
 
 change_shaper(StateData, JID) ->
-    Shaper = acl:match_rule(StateData#state.shaper, JID),
+    Shaper = acl:match_rule(StateData#state.server,
+			    StateData#state.shaper, JID),
     ejabberd_receiver:change_shaper(StateData#state.receiver, Shaper).
 
 send_text(StateData, Text) ->
@@ -1208,6 +1197,7 @@ presence_update(From, Packet, StateData) ->
 		if
 		    FromUnavail ->
 			ejabberd_hooks:run(user_available_hook,
+					   StateData#state.server,
 					   [StateData#state.jid]),
 			resend_offline_messages(StateData),
 			presence_broadcast_first(
@@ -1248,21 +1238,25 @@ presence_track(From, To, Packet, StateData) ->
 	"subscribe" ->
 	    ejabberd_router:route(jlib:jid_remove_resource(From), To, Packet),
 	    ejabberd_hooks:run(roster_out_subscription,
+			       Server,
 			       [User, Server, To, subscribe]),
 	    StateData;
 	"subscribed" ->
 	    ejabberd_router:route(jlib:jid_remove_resource(From), To, Packet),
 	    ejabberd_hooks:run(roster_out_subscription,
+			       Server,
 			       [User, Server, To, subscribed]),
 	    StateData;
 	"unsubscribe" ->
 	    ejabberd_router:route(jlib:jid_remove_resource(From), To, Packet),
 	    ejabberd_hooks:run(roster_out_subscription,
+			       Server,
 			       [User, Server, To, unsubscribe]),
 	    StateData;
 	"unsubscribed" ->
 	    ejabberd_router:route(jlib:jid_remove_resource(From), To, Packet),
 	    ejabberd_hooks:run(roster_out_subscription,
+			       Server,
 			       [User, Server, To, unsubscribed]),
 	    StateData;
 	"error" ->
@@ -1505,7 +1499,9 @@ process_privacy_iq(From, To,
 resend_offline_messages(#state{user = User,
 			       server = Server,
 			       privacy_list = PrivList} = StateData) ->
-    case ejabberd_hooks:run_fold(resend_offline_messages_hook, [],
+    case ejabberd_hooks:run_fold(resend_offline_messages_hook,
+				 Server,
+				 [],
 				 [User, Server]) of
 	Rs when list(Rs) ->
 	    lists:foreach(
@@ -1552,3 +1548,30 @@ get_statustag(Presence) ->
     case xml:get_path_s(Presence, [{elem, "status"}, cdata]) of
 	ShowTag -> ShowTag
     end.
+
+process_unauthenticated_stanza(StateData, El) ->
+    case jlib:iq_query_info(El) of
+	#iq{} = IQ ->
+	    Res = ejabberd_hooks:run_fold(c2s_unauthenticated_iq,
+					  StateData#state.server,
+					  empty,
+					  [StateData#state.server, IQ]),
+	    case Res of
+		empty ->
+		    % The only reasonable IQ's here are auth and register IQ's
+		    % They contain secrets, so don't include subelements to response
+		    ResIQ = IQ#iq{type = error,
+				  sub_el = [?ERR_FEATURE_NOT_IMPLEMENTED]},
+		    Res1 = jlib:replace_from_to(
+			     jlib:make_jid("", StateData#state.server, ""),
+			     jlib:make_jid("", "", ""),
+			     jlib:iq_to_xml(ResIQ)),
+		    send_element(StateData, jlib:remove_attr("to", Res1));
+		_ ->                    
+		    send_element(StateData, Res)
+	    end;
+	_ ->
+	    % Drop any stanza, which isn't IQ stanza
+	    ok
+    end.
+
