@@ -3,12 +3,12 @@
 %%% Author  : Alexey Shchepin <alexey@sevcom.net>
 %%% Purpose : Socket receiver for C2S and S2S connections
 %%% Created : 10 Nov 2003 by Alexey Shchepin <alexey@sevcom.net>
-%%% Id      : $Id: ejabberd_receiver.erl 288 2004-12-03 22:54:02Z aleksey $
+%%% Id      : $Id: ejabberd_receiver.erl 426 2005-10-25 01:08:37Z alexey $
 %%%----------------------------------------------------------------------
 
 -module(ejabberd_receiver).
 -author('alexey@sevcom.net').
--vsn('$Revision$ ').
+-vsn('$Revision: 426 $ ').
 
 -export([start/3,
 	 receiver/4,
@@ -36,24 +36,27 @@ receiver(Socket, SockMod, Shaper, C2SPid) ->
 
 receiver(Socket, SockMod, ShaperState, C2SPid, XMLStreamState, Timeout) ->
     Res = (catch SockMod:recv(Socket, 0, Timeout)),
-    case Res of
-        {ok, Data} ->
-	    receive
-		{starttls, TLSSocket} ->
-		    xml_stream:close(XMLStreamState),
-		    XMLStreamState1 = xml_stream:new(C2SPid),
-		    TLSRes = tls:recv_data(TLSSocket, Data),
-		    receiver1(TLSSocket, tls,
-			      ShaperState, C2SPid, XMLStreamState1, Timeout,
-			      TLSRes)
-	    after 0 ->
-		    receiver1(Socket, SockMod,
-			      ShaperState, C2SPid, XMLStreamState, Timeout,
-			      Res)
-	    end;
-	_ ->
+    receive
+	{starttls, TLSSocket} ->
+	    xml_stream:close(XMLStreamState),
+	    XMLStreamState1 = xml_stream:new(C2SPid),
+	    TLSRes = case Res of
+			 {ok, Data} ->
+			     tls:recv_data(TLSSocket, Data);
+			 _ ->
+			     tls:recv_data(TLSSocket, "")
+		     end,
+	    receiver1(TLSSocket, tls,
+		      ShaperState, C2SPid, XMLStreamState1, Timeout,
+		      TLSRes);
+	{change_timeout, NewTimeout} -> % Dirty hack
 	    receiver1(Socket, SockMod,
-		      ShaperState, C2SPid, XMLStreamState, Timeout, Res)
+		      ShaperState, C2SPid, XMLStreamState, NewTimeout,
+		      Res)
+    after 0 ->
+	    receiver1(Socket, SockMod,
+		      ShaperState, C2SPid, XMLStreamState, Timeout,
+		      Res)
     end.
 
 
