@@ -3,12 +3,12 @@
 %%% Author  : Alexey Shchepin <alexey@sevcom.net>
 %%% Purpose : Main router
 %%% Created : 27 Nov 2002 by Alexey Shchepin <alexey@sevcom.net>
-%%% Id      : $Id: ejabberd_router.erl 307 2005-04-17 18:08:34Z tmallard $
+%%% Id      : $Id: ejabberd_router.erl 381 2005-07-27 00:59:35Z alexey $
 %%%----------------------------------------------------------------------
 
 -module(ejabberd_router).
 -author('alexey@sevcom.net').
--vsn('$Revision$ ').
+-vsn('$Revision: 381 $ ').
 
 -export([route/3,
 	 register_route/1,
@@ -80,45 +80,52 @@ loop() ->
 	    loop()
     end.
 
-do_route(From, To, Packet) ->
-    ?DEBUG("route~n\tfrom ~p~n\tto ~p~n\tpacket ~p~n", [From, To, Packet]),
-    LDstDomain = To#jid.lserver,
-    case mnesia:dirty_read(route, LDstDomain) of
-	[] ->
-	    ejabberd_s2s:route(From, To, Packet);
-	[R] ->
-	    Pid = R#route.pid,
-	    if
-		node(Pid) == node() ->
-		    case R#route.local_hint of
-			{apply, Module, Function} ->
-			    Module:Function(From, To, Packet);
-			_ ->
+do_route(OrigFrom, OrigTo, OrigPacket) ->
+    ?DEBUG("route~n\tfrom ~p~n\tto ~p~n\tpacket ~p~n",
+	   [OrigFrom, OrigTo, OrigPacket]),
+    LOrigDstDomain = OrigTo#jid.lserver,
+    case ejabberd_hooks:run_fold(filter_packet, LOrigDstDomain,
+				 {OrigFrom, OrigTo, OrigPacket}, []) of
+	{From, To, Packet} ->
+	    LDstDomain = To#jid.lserver,
+	    case mnesia:dirty_read(route, LDstDomain) of
+		[] ->
+		    ejabberd_s2s:route(From, To, Packet);
+		[R] ->
+		    Pid = R#route.pid,
+		    if
+			node(Pid) == node() ->
+			    case R#route.local_hint of
+				{apply, Module, Function} ->
+				    Module:Function(From, To, Packet);
+				_ ->
+				    Pid ! {route, From, To, Packet}
+			    end;
+			true ->
 			    Pid ! {route, From, To, Packet}
 		    end;
-		true ->
-		    Pid ! {route, From, To, Packet}
-	    end;
-	Rs ->
-	    case [R || R <- Rs, node(R#route.pid) == node()] of
-		[] ->
-		    R = lists:nth(erlang:phash(now(), length(Rs)), Rs),
-		    Pid = R#route.pid,
-		    Pid ! {route, From, To, Packet};
-		LRs ->
-		    LRs,
-		    R = lists:nth(erlang:phash(now(), length(LRs)), LRs),
-		    Pid = R#route.pid,
-		    case R#route.local_hint of
-			{apply, Module, Function} ->
-			    Module:Function(From, To, Packet);
-			_ ->
-			    Pid ! {route, From, To, Packet}
+		Rs ->
+		    case [R || R <- Rs, node(R#route.pid) == node()] of
+			[] ->
+			    R = lists:nth(erlang:phash(now(), length(Rs)), Rs),
+			    Pid = R#route.pid,
+			    Pid ! {route, From, To, Packet};
+			LRs ->
+			    LRs,
+			    R = lists:nth(erlang:phash(now(), length(LRs)), LRs),
+			    Pid = R#route.pid,
+			    case R#route.local_hint of
+				{apply, Module, Function} ->
+				    Module:Function(From, To, Packet);
+				_ ->
+				    Pid ! {route, From, To, Packet}
+			    end
 		    end
-	    end
+	    end;
+	drop ->
+	    ok
     end.
 
-
 %route(From, To, Packet) ->
 %    ejabberd_router ! {route, From, To, Packet}.
 
