@@ -3,12 +3,12 @@
 %%% Author  : Alexey Shchepin <alexey@sevcom.net>
 %%% Purpose : S2S connections manager
 %%% Created :  7 Dec 2002 by Alexey Shchepin <alexey@sevcom.net>
-%%% Id      : $Id: ejabberd_s2s.erl 285 2004-11-05 21:14:31Z aleksey $
+%%% Id      : $Id: ejabberd_s2s.erl 433 2005-11-12 03:42:12Z alexey $
 %%%----------------------------------------------------------------------
 
 -module(ejabberd_s2s).
 -author('alexey@sevcom.net').
--vsn('$Revision$ ').
+-vsn('$Revision: 433 $ ').
 
 -export([start_link/0, init/0,
 	 route/3,
@@ -167,7 +167,9 @@ find_connection(From, To) ->
 				Pid
 			end
 		end,
-	    mnesia:transaction(F);
+	    TRes = mnesia:transaction(F),
+	    ejabberd_s2s_out:start_connection(Pid),
+	    TRes;
 	[El] ->
 	    {atomic, El#s2s.pid}
     end.
