@@ -3,12 +3,12 @@
 %%% Author  : Alexey Shchepin <alexey@sevcom.net>
 %%% Purpose : 
 %%% Created :  6 Dec 2002 by Alexey Shchepin <alexey@sevcom.net>
-%%% Id      : $Id: ejabberd_service.erl 198 2004-01-11 20:42:57Z aleksey $
+%%% Id      : $Id: ejabberd_service.erl 458 2005-12-10 18:42:08Z alexey $
 %%%----------------------------------------------------------------------
 
 -module(ejabberd_service).
 -author('alexey@sevcom.net').
--vsn('$Revision$ ').
+-vsn('$Revision: 458 $ ').
 
 -behaviour(gen_fsm).
 
@@ -283,7 +283,7 @@ handle_info({send_element, El}, StateName, StateData) ->
     send_element(StateData, El),
     {next_state, StateName, StateData};
 handle_info({route, From, To, Packet}, StateName, StateData) ->
-    case acl:match_rule(StateData#state.access, From) of
+    case acl:match_rule(global, StateData#state.access, From) of
 	allow ->
 	    {xmlelement, Name, Attrs, Els} = Packet,
 	    Attrs2 = jlib:replace_from_to_attrs(jlib:jid_to_string(From),
@@ -303,7 +303,8 @@ handle_info({route, From, To, Packet}, StateName, StateData) ->
 %% Purpose: Shutdown the fsm
 %% Returns: any
 %%----------------------------------------------------------------------
-terminate(_Reason, StateName, StateData) ->
+terminate(Reason, StateName, StateData) ->
+    ?INFO_MSG("terminated: ~p", [Reason]),
     case StateName of
 	stream_established ->
 	    lists:foreach(
