@@ -3,26 +3,26 @@
 %%% Author  : Alexey Shchepin <alexey@sevcom.net>
 %%% Purpose : 
 %%% Created : 15 Jan 2003 by Alexey Shchepin <alexey@sevcom.net>
-%%% Id      : $Id: mod_echo.erl 354 2005-05-23 00:30:29Z alexey $
+%%% Id      : $Id: mod_echo.erl 370 2005-06-20 03:18:13Z alexey $
 %%%----------------------------------------------------------------------
 
 -module(mod_echo).
 -author('alexey@sevcom.net').
--vsn('$Revision: 354 $ ').
+-vsn('$Revision: 370 $ ').
 
 -behaviour(gen_mod).
 
--export([start/1, init/1, stop/0]).
+-export([start/2, init/1, stop/1]).
 
 -include("ejabberd.hrl").
 -include("jlib.hrl").
 
 -define(PROCNAME, ejabberd_mod_echo).
 
-start(Opts) ->
-    %Host = gen_mod:get_opt(host, Opts),
-    Host = gen_mod:get_opt(host, Opts, "echo." ++ ?MYNAME),
-    register(?PROCNAME, spawn(?MODULE, init, [Host])).
+start(Host, Opts) ->
+    MyHost = gen_mod:get_opt(host, Opts, "echo." ++ Host),
+    register(gen_mod:get_module_proc(Host, ?PROCNAME),
+	     spawn(?MODULE, init, [MyHost])).
 
 init(Host) ->
     ejabberd_router:register_route(Host),
@@ -40,7 +40,8 @@ loop(Host) ->
 	    loop(Host)
     end.
 
-stop() ->
-    ?PROCNAME ! stop,
-    {wait, ?PROCNAME}.
+stop(Host) ->
+    Proc = gen_mod:get_module_proc(Host, ?PROCNAME),
+    Proc ! stop,
+    {wait, Proc}.
 
