@@ -3,12 +3,12 @@
 %%% Author  : Alexey Shchepin <alexey@sevcom.net>
 %%% Purpose : 
 %%% Created : 15 Feb 2003 by Alexey Shchepin <alexey@sevcom.net>
-%%% Id      : $Id: mod_irc_connection.erl,v 1.20 2004/08/02 22:17:05 aleksey Exp $
+%%% Id      : $Id: mod_irc_connection.erl 349 2005-05-17 22:35:16Z alexey $
 %%%----------------------------------------------------------------------
 
 -module(mod_irc_connection).
 -author('alexey@sevcom.net').
--vsn('$Revision: 1.20 $ ').
+-vsn('$Revision: 349 $ ').
 
 -behaviour(gen_fsm).
 
@@ -32,7 +32,7 @@
 -define(SETS, gb_sets).
 
 -record(state, {socket, encoding, receiver, queue,
-		user, myname, server, nick,
+		user, host, server, nick,
 		channels = dict:new(),
 		inbuf = "", outbuf = ""}).
 
@@ -67,7 +67,7 @@ init([From, Host, Server, Username, Encoding]) ->
 			     encoding = Encoding,
 			     user = From,
 			     nick = Username,
-			     myname = Host,
+			     host = Host,
 			     server = Server}}.
 
 %%----------------------------------------------------------------------
@@ -90,7 +90,7 @@ open_socket(init, StateData) ->
 			"USER ~s ~s ~s :~s\r\n",
 			[StateData#state.nick,
 			 StateData#state.nick,
-			 StateData#state.myname,
+			 StateData#state.host,
 			 StateData#state.nick])),
 	    send_text(NewStateData,
 		      io_lib:format("CODEPAGE ~s\r\n", [StateData#state.encoding])),
@@ -231,7 +231,7 @@ handle_info({route_chan, Channel, Resource,
 			  jlib:make_jid(
 			    lists:concat(
 			      [Channel, "%", StateData#state.server]),
-			    StateData#state.myname, StateData#state.nick),
+			    StateData#state.host, StateData#state.nick),
 			  StateData#state.user, El),
 			Body = xml:get_path_s(El, [{elem, "body"}, cdata]),
 			case Body of
@@ -304,7 +304,7 @@ handle_info({route_chan, Channel, Resource,
 	    StateName, StateData) ->
     From = StateData#state.user,
     To = jlib:make_jid(lists:concat([Channel, "%", StateData#state.server]),
-		       StateData#state.myname, StateData#state.nick),
+		       StateData#state.host, StateData#state.nick),
     case jlib:iq_query_info(El) of
 	#iq{xmlns = ?NS_MUC_ADMIN} = IQ ->
 	    iq_admin(StateData, Channel, From, To, IQ);
@@ -322,6 +322,13 @@ handle_info({route_chan, Channel, Resource,
 	    Err = jlib:make_error_reply(
 		    El, ?ERR_FEATURE_NOT_IMPLEMENTED),
 	    ejabberd_router:route(To, From, Err);
+	#iq{xmlns = ?NS_VCARD} ->
+	    Res = io_lib:format("WHOIS ~s \r\n",
+				[Resource]),
+	    ?SEND(Res),
+	    Err = jlib:make_error_reply(
+		    El, ?ERR_FEATURE_NOT_IMPLEMENTED),
+	    ejabberd_router:route(To, From, Err);
 	#iq{} ->
 	    Err = jlib:make_error_reply(
 		    El, ?ERR_FEATURE_NOT_IMPLEMENTED),
@@ -390,6 +397,18 @@ handle_info({ircstring, [$: | String]}, StateName, StateData) ->
 	    [_, "332", Nick, [$# | Chan] | _] ->
 		process_channel_topic(StateData, Chan, String),
 		StateData;
+	    [_, "318", _, Nick | _] ->
+		process_endofwhois(StateData, String, Nick),
+		StateData;
+	    [_, "311", _, Nick, Ident, Irchost | _ ] ->
+		process_whois311(StateData, String, Nick, Ident, Irchost),
+		StateData;
+	    [_, "312", _, Nick, Ircserver  | _ ] ->
+		process_whois312(StateData, String, Nick, Ircserver),
+		StateData;
+	    [_, "319", _, Nick | _ ] ->
+		process_whois319(StateData, String, Nick),
+		StateData;
 	    [From, "PRIVMSG", [$# | Chan] | _] ->
 		process_chanprivmsg(StateData, Chan, From, String),
 		StateData;
@@ -399,6 +418,9 @@ handle_info({ircstring, [$: | String]}, StateName, StateData) ->
 	    [From, "PRIVMSG", Nick, ":\001VERSION\001" | _] ->
 		process_version(StateData, Nick, From),
 		StateData;
+	    [From, "PRIVMSG", Nick, ":\001USERINFO\001" | _] ->
+		process_userinfo(StateData, Nick, From),
+		StateData;
 	    [From, "PRIVMSG", Nick | _] ->
 		process_privmsg(StateData, Nick, From, String),
 		StateData;
@@ -474,15 +496,16 @@ handle_info({tcp_error, Socket, Reason}, StateName, StateData) ->
 %% Returns: any
 %%----------------------------------------------------------------------
 terminate(Reason, StateName, StateData) ->
-    mod_irc:closed_conection(StateData#state.user,
-			     StateData#state.server),
+    mod_irc:closed_connection(StateData#state.host,
+			      StateData#state.user,
+			      StateData#state.server),
     bounce_messages("Server Connect Failed"),
     lists:foreach(
       fun(Chan) ->
 	      ejabberd_router:route(
 		jlib:make_jid(
 		  lists:concat([Chan, "%", StateData#state.server]),
-		  StateData#state.myname, StateData#state.nick),
+		  StateData#state.host, StateData#state.nick),
 		StateData#state.user,
 		{xmlelement, "presence", [{"type", "error"}],
 		 [{xmlelement, "error", [{"code", "502"}],
@@ -592,7 +615,7 @@ process_channel_list_user(StateData, Chan, User) ->
 	end,
     ejabberd_router:route(
       jlib:make_jid(lists:concat([Chan, "%", StateData#state.server]),
-		    StateData#state.myname, User2),
+		    StateData#state.host, User2),
       StateData#state.user,
       {xmlelement, "presence", [],
        [{xmlelement, "x", [{"xmlns", ?NS_MUC_USER}],
@@ -614,25 +637,60 @@ process_channel_list_user(StateData, Chan, User) ->
 process_channel_topic(StateData, Chan, String) ->
     FromUser = "someone",
     {ok, Msg, _} = regexp:sub(String, ".*332[^:]*:", ""),
-    Msg1 = lists:filter(
-	     fun(C) ->
-		     if (C < 32) and
-			(C /= 9) and
-			(C /= 10) and
-			(C /= 13) ->
-			     false;
-			true -> true
-		     end
-	     end, Msg),
+    Msg1 = filter_message(Msg),
     ejabberd_router:route(
       jlib:make_jid(
 	lists:concat([Chan, "%", StateData#state.server]),
-	StateData#state.myname, FromUser),
+	StateData#state.host, FromUser),
       StateData#state.user,
       {xmlelement, "message", [{"type", "groupchat"}],
        [{xmlelement, "subject", [], [{xmlcdata, Msg1}]}]}).
 
 
+process_endofwhois(StateData, String, Nick) ->
+    ejabberd_router:route(
+      jlib:make_jid(lists:concat([Nick, "!", StateData#state.server]),
+		    StateData#state.host, ""),
+      StateData#state.user,
+      {xmlelement, "message", [{"type", "chat"}],
+       [{xmlelement, "body", [], [{xmlcdata, "End of WHOIS"}]}]}).
+
+process_whois311(StateData, String, Nick, Ident, Irchost) ->
+    {ok, Fullname, _} = regexp:sub(String, ".*311[^:]*:", ""),
+    ejabberd_router:route(
+      jlib:make_jid(lists:concat([Nick, "!", StateData#state.server]),
+		    StateData#state.host, ""),
+      StateData#state.user,
+      {xmlelement, "message", [{"type", "chat"}],
+       [{xmlelement, "body", [],
+	 [{xmlcdata, lists:concat(
+		       ["WHOIS: ", Nick, " is ",
+			Ident, "@" , Irchost, " : " , Fullname])}]}]}).
+
+process_whois312(StateData, String, Nick, Ircserver) ->
+    {ok, Ircserverdesc, _} = regexp:sub(String, ".*312[^:]*:", ""),
+    ejabberd_router:route(
+      jlib:make_jid(lists:concat([Nick, "!", StateData#state.server]),
+		    StateData#state.host, ""),
+      StateData#state.user,
+      {xmlelement, "message", [{"type", "chat"}],
+       [{xmlelement, "body", [],
+	 [{xmlcdata, lists:concat(["WHOIS: ", Nick, " use ",
+				   Ircserver, " : ", Ircserverdesc])}]}]}).
+
+process_whois319(StateData, String, Nick) ->
+    {ok, Chanlist, _} = regexp:sub(String, ".*319[^:]*:", ""),
+    ejabberd_router:route(
+      jlib:make_jid(lists:concat([Nick, "!", StateData#state.server]),
+		    StateData#state.host, ""),
+      StateData#state.user,
+      {xmlelement, "message", [{"type", "chat"}],
+       [{xmlelement, "body", [],
+	 [{xmlcdata, lists:concat(["WHOIS: ", Nick, " is on ",
+				   Chanlist])}]}]}).
+
+
+
 process_chanprivmsg(StateData, Chan, From, String) ->
     [FromUser | _] = string:tokens(From, "!"),
     {ok, Msg, _} = regexp:sub(String, ".*PRIVMSG[^:]*:", ""),
@@ -642,19 +700,10 @@ process_chanprivmsg(StateData, Chan, From, String) ->
 	       _ ->
 		   Msg
 	   end,
-    Msg2 = lists:filter(
-	     fun(C) ->
-		     if (C < 32) and
-			(C /= 9) and
-			(C /= 10) and
-			(C /= 13) ->
-			     false;
-			true -> true
-		     end
-	     end, Msg1),
+    Msg2 = filter_message(Msg1),
     ejabberd_router:route(
       jlib:make_jid(lists:concat([Chan, "%", StateData#state.server]),
-		    StateData#state.myname, FromUser),
+		    StateData#state.host, FromUser),
       StateData#state.user,
       {xmlelement, "message", [{"type", "groupchat"}],
        [{xmlelement, "body", [], [{xmlcdata, Msg2}]}]}).
@@ -670,19 +719,10 @@ process_channotice(StateData, Chan, From, String) ->
 	       _ ->
 		   Msg
 	   end,
-    Msg2 = lists:filter(
-	     fun(C) ->
-		     if (C < 32) and
-			(C /= 9) and
-			(C /= 10) and
-			(C /= 13) ->
-			     false;
-			true -> true
-		     end
-	     end, Msg1),
+    Msg2 = filter_message(Msg1),
     ejabberd_router:route(
       jlib:make_jid(lists:concat([Chan, "%", StateData#state.server]),
-		    StateData#state.myname, FromUser),
+		    StateData#state.host, FromUser),
       StateData#state.user,
       {xmlelement, "message", [{"type", "groupchat"}],
        [{xmlelement, "body", [], [{xmlcdata, "NOTICE: " ++ Msg2}]}]}).
@@ -699,19 +739,10 @@ process_privmsg(StateData, Nick, From, String) ->
 	       _ ->
 		   Msg
 	   end,
-    Msg2 = lists:filter(
-	     fun(C) ->
-		     if (C < 32) and
-			(C /= 9) and
-			(C /= 10) and
-			(C /= 13) ->
-			     false;
-			true -> true
-		     end
-	     end, Msg1),
+    Msg2 = filter_message(Msg1),
     ejabberd_router:route(
       jlib:make_jid(lists:concat([FromUser, "!", StateData#state.server]),
-		    StateData#state.myname, ""),
+		    StateData#state.host, ""),
       StateData#state.user,
       {xmlelement, "message", [{"type", "chat"}],
        [{xmlelement, "body", [], [{xmlcdata, Msg2}]}]}).
@@ -726,19 +757,10 @@ process_notice(StateData, Nick, From, String) ->
 	       _ ->
 		   Msg
 	   end,
-    Msg2 = lists:filter(
-	     fun(C) ->
-		     if (C < 32) and
-			(C /= 9) and
-			(C /= 10) and
-			(C /= 13) ->
-			     false;
-			true -> true
-		     end
-	     end, Msg1),
+    Msg2 = filter_message(Msg1),
     ejabberd_router:route(
       jlib:make_jid(lists:concat([FromUser, "!", StateData#state.server]),
-		    StateData#state.myname, ""),
+		    StateData#state.host, ""),
       StateData#state.user,
       {xmlelement, "message", [{"type", "chat"}],
        [{xmlelement, "body", [], [{xmlcdata, "NOTICE: " ++ Msg2}]}]}).
@@ -758,22 +780,24 @@ process_version(StateData, Nick, From) ->
 		    [FromUser])).
 
 
+process_userinfo(StateData, Nick, From) ->
+    [FromUser | _] = string:tokens(From, "!"),
+    send_text(
+      StateData,
+      io_lib:format("NOTICE ~s :\001USERINFO "
+		    "This user uses xmpp:~s"
+		    "\001\r\n",
+		    [FromUser,
+		     jlib:jid_to_string(StateData#state.user)])).
+
+
 process_topic(StateData, Chan, From, String) ->
     [FromUser | _] = string:tokens(From, "!"),
     {ok, Msg, _} = regexp:sub(String, ".*TOPIC[^:]*:", ""),
-    Msg1 = lists:filter(
-	     fun(C) ->
-		     if (C < 32) and
-			(C /= 9) and
-			(C /= 10) and
-			(C /= 13) ->
-			     false;
-			true -> true
-		     end
-	     end, Msg),
+    Msg1 = filter_message(Msg),
     ejabberd_router:route(
       jlib:make_jid(lists:concat([Chan, "%", StateData#state.server]),
-		    StateData#state.myname, FromUser),
+		    StateData#state.host, FromUser),
       StateData#state.user,
       {xmlelement, "message", [{"type", "groupchat"}],
        [{xmlelement, "subject", [], [{xmlcdata, Msg1}]},
@@ -784,18 +808,19 @@ process_topic(StateData, Chan, From, String) ->
 process_part(StateData, Chan, From, String) ->
     [FromUser | FromIdent] = string:tokens(From, "!"),
     {ok, Msg, _} = regexp:sub(String, ".*PART[^:]*", ""),    
+    Msg1 = filter_message(Msg),
     ejabberd_router:route(
       jlib:make_jid(lists:concat([Chan, "%", StateData#state.server]),
-		    StateData#state.myname, FromUser),
+		    StateData#state.host, FromUser),
       StateData#state.user,
       {xmlelement, "message", [{"type", "groupchat"}],
        [{xmlelement, "body", [],
 	 [{xmlcdata, "/me has part: " ++
-	   Msg ++ "("  ++ FromIdent ++ ")" }]}]}),
+	   Msg1 ++ "("  ++ FromIdent ++ ")" }]}]}),
 
     ejabberd_router:route(
       jlib:make_jid(lists:concat([Chan, "%", StateData#state.server]),
-		    StateData#state.myname, FromUser),
+		    StateData#state.host, FromUser),
       StateData#state.user,
       {xmlelement, "presence", [{"type", "unavailable"}],
        [{xmlelement, "x", [{"xmlns", ?NS_MUC_USER}],
@@ -804,7 +829,7 @@ process_part(StateData, Chan, From, String) ->
 	    {"role", "none"}],
 	   []}]},
 	{xmlelement, "status", [],
-	 [{xmlcdata, Msg ++ "("  ++ FromIdent ++ ")"}]}]
+	 [{xmlcdata, Msg1 ++ "("  ++ FromIdent ++ ")"}]}]
       }),
     case catch dict:update(Chan,
 			   fun(Ps) ->
@@ -821,6 +846,7 @@ process_quit(StateData, From, String) ->
     [FromUser | FromIdent] = string:tokens(From, "!"),
     
     {ok, Msg, _} = regexp:sub(String, ".*QUIT[^:]*:", ""),
+    Msg1 = filter_message(Msg),
     NewChans =
 	dict:map(
 	  fun(Chan, Ps) ->
@@ -829,17 +855,17 @@ process_quit(StateData, From, String) ->
 		          ejabberd_router:route(
 			    jlib:make_jid(
 			      lists:concat([Chan, "%", StateData#state.server]),
-			      StateData#state.myname, FromUser),
+			      StateData#state.host, FromUser),
 			    StateData#state.user,
 			    {xmlelement, "message", [{"type", "groupchat"}],
 			     [{xmlelement, "body", [],
 			       [{xmlcdata, "/me has quit: " ++
-				 Msg ++ "("  ++ FromIdent ++ ")" }]}]}),
+				 Msg1 ++ "("  ++ FromIdent ++ ")" }]}]}),
 
 			  ejabberd_router:route(
 			    jlib:make_jid(
 			      lists:concat([Chan, "%", StateData#state.server]),
-			      StateData#state.myname, FromUser),
+			      StateData#state.host, FromUser),
 			    StateData#state.user,
 			    {xmlelement, "presence", [{"type", "unavailable"}],
 			     [{xmlelement, "x", [{"xmlns", ?NS_MUC_USER}],
@@ -848,7 +874,7 @@ process_quit(StateData, From, String) ->
 				  {"role", "none"}],
 				 []}]},
 			      {xmlelement, "status", [],
-			       [{xmlcdata, Msg ++ "("  ++ FromIdent ++ ")"}]}
+			       [{xmlcdata, Msg1 ++ "("  ++ FromIdent ++ ")"}]}
 			     ]}),
 			  remove_element(FromUser, Ps);
 		      _ ->
@@ -863,7 +889,7 @@ process_join(StateData, Channel, From, String) ->
     Chan = lists:subtract(Channel, ":#"),
     ejabberd_router:route(
       jlib:make_jid(lists:concat([Chan, "%", StateData#state.server]),
-		    StateData#state.myname, FromUser),
+		    StateData#state.host, FromUser),
       StateData#state.user,
       {xmlelement, "presence", [],
        [{xmlelement, "x", [{"xmlns", ?NS_MUC_USER}],
@@ -874,14 +900,15 @@ process_join(StateData, Channel, From, String) ->
 	{xmlelement, "status", [],
 	 [{xmlcdata, FromIdent}]}]}),
     {ok, Msg, _} = regexp:sub(String, ".*JOIN[^:]*:", ""),    
+    Msg1 = filter_message(Msg),
     ejabberd_router:route(
       jlib:make_jid(lists:concat([Chan, "%", StateData#state.server]),
-		    StateData#state.myname, FromUser),
+		    StateData#state.host, FromUser),
       StateData#state.user,
       {xmlelement, "message", [{"type", "groupchat"}],
        [{xmlelement, "body", [],
 	 [{xmlcdata, "/me has joined " ++
-	   Msg ++ "("  ++ FromIdent ++ ")" }]}]}),
+	   Msg1 ++ "("  ++ FromIdent ++ ")" }]}]}),
 
     case catch dict:update(Chan,
 			   fun(Ps) ->
@@ -899,7 +926,7 @@ process_mode_o(StateData, Chan, From, Nick, Affiliation, Role) ->
     %Msg = lists:last(string:tokens(String, ":")),
     ejabberd_router:route(
       jlib:make_jid(lists:concat([Chan, "%", StateData#state.server]),
-		    StateData#state.myname, Nick),
+		    StateData#state.host, Nick),
       StateData#state.user,
       {xmlelement, "presence", [],
        [{xmlelement, "x", [{"xmlns", ?NS_MUC_USER}],
@@ -912,7 +939,7 @@ process_kick(StateData, Chan, From, Nick) ->
     %Msg = lists:last(string:tokens(String, ":")),
     ejabberd_router:route(
       jlib:make_jid(lists:concat([Chan, "%", StateData#state.server]),
-		    StateData#state.myname, Nick),
+		    StateData#state.host, Nick),
       StateData#state.user,
       {xmlelement, "presence", [{"type", "unavailable"}],
        [{xmlelement, "x", [{"xmlns", ?NS_MUC_USER}],
@@ -934,7 +961,7 @@ process_nick(StateData, From, NewNick) ->
 			  ejabberd_router:route(
 			    jlib:make_jid(
 			      lists:concat([Chan, "%", StateData#state.server]),
-			      StateData#state.myname, FromUser),
+			      StateData#state.host, FromUser),
 			    StateData#state.user,
 			    {xmlelement, "presence", [{"type", "unavailable"}],
 			     [{xmlelement, "x", [{"xmlns", ?NS_MUC_USER}],
@@ -948,7 +975,7 @@ process_nick(StateData, From, NewNick) ->
 			  ejabberd_router:route(
 			    jlib:make_jid(
 			      lists:concat([Chan, "%", StateData#state.server]),
-			      StateData#state.myname, Nick),
+			      StateData#state.host, Nick),
 			    StateData#state.user,
 			    {xmlelement, "presence", [],
 			     [{xmlelement, "x", [{"xmlns", ?NS_MUC_USER}],
@@ -972,7 +999,7 @@ process_error(StateData, String) ->
 	      ejabberd_router:route(
 		jlib:make_jid(
 		  lists:concat([Chan, "%", StateData#state.server]),
-		  StateData#state.myname, StateData#state.nick),
+		  StateData#state.host, StateData#state.nick),
 		StateData#state.user,
 		{xmlelement, "presence", [{"type", "error"}],
 		 [{xmlelement, "error", [{"code", "502"}],
@@ -1058,7 +1085,14 @@ process_admin(StateData, Channel, Nick, Affiliation, Role, Reason) ->
 
 
 
-
-
-
-
+filter_message(Msg) ->
+    lists:filter(
+      fun(C) ->
+	      if (C < 32) and
+		 (C /= 9) and
+		 (C /= 10) and
+		 (C /= 13) ->
+		      false;
+		 true -> true
+	      end
+      end, Msg).
