@@ -3,45 +3,45 @@
 %%% Author  : Alexey Shchepin <alexey@sevcom.net>
 %%% Purpose : jabber:iq:last support (JEP-0012)
 %%% Created : 24 Oct 2003 by Alexey Shchepin <alexey@sevcom.net>
-%%% Id      : $Id: mod_last_odbc.erl 358 2005-05-23 19:47:57Z alexey $
+%%% Id      : $Id: mod_last_odbc.erl 379 2005-07-26 03:04:26Z alexey $
 %%%----------------------------------------------------------------------
 
 -module(mod_last_odbc).
 -author('alexey@sevcom.net').
--vsn('$Revision: 358 $ ').
+-vsn('$Revision: 379 $ ').
 
 -behaviour(gen_mod).
 
--export([start/1,
-	 stop/0,
+-export([start/2,
+	 stop/1,
 	 process_local_iq/3,
 	 process_sm_iq/3,
 	 on_presence_update/4,
 	 store_last_info/4,
-	 remove_user/1]).
+	 remove_user/2]).
 
 -include("ejabberd.hrl").
 -include("jlib.hrl").
 
 
-start(Opts) ->
+start(Host, Opts) ->
     IQDisc = gen_mod:get_opt(iqdisc, Opts, one_queue),
-    gen_iq_handler:add_iq_handler(ejabberd_local, ?NS_LAST,
+    gen_iq_handler:add_iq_handler(ejabberd_local, Host, ?NS_LAST,
 				  ?MODULE, process_local_iq, IQDisc),
-    gen_iq_handler:add_iq_handler(ejabberd_sm, ?NS_LAST,
+    gen_iq_handler:add_iq_handler(ejabberd_sm, Host, ?NS_LAST,
 				  ?MODULE, process_sm_iq, IQDisc),
-    ejabberd_hooks:add(remove_user,
+    ejabberd_hooks:add(remove_user, Host,
 		       ?MODULE, remove_user, 50),
-    ejabberd_hooks:add(unset_presence_hook,
+    ejabberd_hooks:add(unset_presence_hook, Host,
 		       ?MODULE, on_presence_update, 50).
 
-stop() ->
-    ejabberd_hooks:delete(remove_user,
+stop(Host) ->
+    ejabberd_hooks:delete(remove_user, Host,
 			  ?MODULE, remove_user, 50),
-    ejabberd_hooks:delete(unset_presence_hook,
+    ejabberd_hooks:delete(unset_presence_hook, Host,
 			  ?MODULE, on_presence_update, 50),
-    gen_iq_handler:remove_iq_handler(ejabberd_local, ?NS_LAST),
-    gen_iq_handler:remove_iq_handler(ejabberd_sm, ?NS_LAST).
+    gen_iq_handler:remove_iq_handler(ejabberd_local, Host, ?NS_LAST),
+    gen_iq_handler:remove_iq_handler(ejabberd_sm, Host, ?NS_LAST).
 
 process_local_iq(_From, _To, #iq{type = Type, sub_el = SubEl} = IQ) ->
     case Type of
@@ -66,22 +66,23 @@ process_sm_iq(From, To, #iq{type = Type, sub_el = SubEl} = IQ) ->
 	    Server = To#jid.lserver,
 	    {Subscription, _Groups} =
 		ejabberd_hooks:run_fold(
-		  roster_get_jid_info, {none, []}, [User, From]),
+		  roster_get_jid_info, Server,
+		  {none, []}, [User, Server, From]),
 	    if
 		(Subscription == both) or (Subscription == from) ->
 		    case catch mod_privacy:get_user_list(User, Server) of
 			{'EXIT', _Reason} ->
-			    get_last(IQ, SubEl, User);
+			    get_last(IQ, SubEl, User, Server);
 			List ->
 			    case catch mod_privacy:check_packet(
-					 User, ?MYNAME, List, % TODO
+					 User, Server, List,
 					 {From, To,
 					  {xmlelement, "presence", [], []}},
 					 out) of
 				{'EXIT', _Reason} ->
-				    get_last(IQ, SubEl, User);
+				    get_last(IQ, SubEl, User, Server);
 				allow ->
-				    get_last(IQ, SubEl, User);
+				    get_last(IQ, SubEl, User, Server);
 				deny ->
 				    IQ#iq{type = error,
 					  sub_el = [SubEl, ?ERR_NOT_ALLOWED]}
@@ -93,9 +94,10 @@ process_sm_iq(From, To, #iq{type = Type, sub_el = SubEl} = IQ) ->
 	    end
     end.
 
-get_last(IQ, SubEl, LUser) ->
+get_last(IQ, SubEl, LUser, LServer) ->
     Username = ejabberd_odbc:escape(LUser),
     case catch ejabberd_odbc:sql_query(
+		 LServer,
 		 ["select seconds, state from last "
 		  "where username='", Username, "'"]) of
 	{'EXIT', _Reason} ->
@@ -128,10 +130,12 @@ on_presence_update(User, Server, _Resource, Status) ->
 
 store_last_info(User, Server, TimeStamp, Status) ->
     LUser = jlib:nodeprep(User),
+    LServer = jlib:nameprep(Server),
     Username = ejabberd_odbc:escape(LUser),
     Seconds = ejabberd_odbc:escape(integer_to_list(TimeStamp)),
     State = ejabberd_odbc:escape(Status),
     ejabberd_odbc:sql_query(
+      LServer,
       ["begin;"
        "delete from last where username='", Username, "';"
        "insert into last(username, seconds, state) "
@@ -139,9 +143,11 @@ store_last_info(User, Server, TimeStamp, Status) ->
        "commit"]).
 
 
-remove_user(User) ->
+remove_user(User, Server) ->
     LUser = jlib:nodeprep(User),
+    LServer = jlib:nameprep(Server),
     Username = ejabberd_odbc:escape(LUser),
     ejabberd_odbc:sql_query(
+      LServer,
       ["delete from last where username='", Username, "'"]).
 
