@@ -3,7 +3,7 @@
 %%% Author  : Alexey Shchepin <alexey@sevcom.net>
 %%% Purpose : 
 %%% Created :  5 Jan 2003 by Alexey Shchepin <alexey@sevcom.net>
-%%% Id      : $Id: mod_offline.erl 354 2005-05-23 00:30:29Z alexey $
+%%% Id      : $Id: mod_offline.erl 454 2005-12-07 13:01:33Z alexey $
 %%%----------------------------------------------------------------------
 
 -module(mod_offline).
@@ -11,9 +11,9 @@
 
 -behaviour(gen_mod).
 
--export([start/1,
+-export([start/2,
 	 init/0,
-	 stop/0,
+	 stop/1,
 	 store_packet/3,
 	 resend_offline_messages/2,
 	 pop_offline_messages/3,
@@ -29,21 +29,22 @@
 -define(PROCNAME, ejabberd_offline).
 -define(OFFLINE_TABLE_LOCK_THRESHOLD, 1000).
 
-start(_) ->
+start(Host, _Opts) ->
     mnesia:create_table(offline_msg,
 			[{disc_only_copies, [node()]},
 			 {type, bag},
 			 {attributes, record_info(fields, offline_msg)}]),
     update_table(),
-    ejabberd_hooks:add(offline_message_hook,
+    ejabberd_hooks:add(offline_message_hook, Host,
 		       ?MODULE, store_packet, 50),
-    ejabberd_hooks:add(offline_subscription_hook,
+    ejabberd_hooks:add(offline_subscription_hook, Host,
 		       ?MODULE, store_packet, 50),
-    ejabberd_hooks:add(resend_offline_messages_hook,
+    ejabberd_hooks:add(resend_offline_messages_hook, Host,
 		       ?MODULE, pop_offline_messages, 50),
-    ejabberd_hooks:add(remove_user,
+    ejabberd_hooks:add(remove_user, Host,
 		       ?MODULE, remove_user, 50),
-    register(?PROCNAME, spawn(?MODULE, init, [])).
+    register(gen_mod:get_module_proc(Host, ?PROCNAME),
+	     spawn(?MODULE, init, [])).
 
 init() ->
     loop().
@@ -79,34 +80,37 @@ receive_all(Msgs) ->
     end.
 
 
-stop() ->
-    ejabberd_hooks:delete(offline_message_hook,
+stop(Host) ->
+    ejabberd_hooks:delete(offline_message_hook, Host,
 			  ?MODULE, store_packet, 50),
-    ejabberd_hooks:delete(offline_subscription_hook,
+    ejabberd_hooks:delete(offline_subscription_hook, Host,
 			  ?MODULE, store_packet, 50),
-    ejabberd_hooks:delete(resend_offline_messages_hook,
+    ejabberd_hooks:delete(resend_offline_messages_hook, Host,
 			  ?MODULE, pop_offline_messages, 50),
-    ejabberd_hooks:delete(remove_user,
+    ejabberd_hooks:delete(remove_user, Host,
 			  ?MODULE, remove_user, 50),
-    exit(whereis(?PROCNAME), stop),
-    {wait, ?PROCNAME}.
+    Proc = gen_mod:get_module_proc(Host, ?PROCNAME),
+    exit(whereis(Proc), stop),
+    {wait, Proc}.
 
 store_packet(From, To, Packet) ->
     Type = xml:get_tag_attr_s("type", Packet),
     if
-	(Type /= "error") and (Type /= "groupchat") ->
+	(Type /= "error") and (Type /= "groupchat") and
+	(Type /= "headline") ->
 	    case check_event(From, To, Packet) of
 		true ->
 		    #jid{luser = LUser, lserver = LServer} = To,
 		    TimeStamp = now(),
 		    {xmlelement, _Name, _Attrs, Els} = Packet,
 		    Expire = find_x_expire(TimeStamp, Els),
-		    ?PROCNAME ! #offline_msg{us = {LUser, LServer},
-					     timestamp = TimeStamp,
-					     expire = Expire,
-					     from = From,
-					     to = To,
-					     packet = Packet},
+		    gen_mod:get_module_proc(To#jid.lserver, ?PROCNAME) !
+			#offline_msg{us = {LUser, LServer},
+				     timestamp = TimeStamp,
+				     expire = Expire,
+				     from = From,
+				     to = To,
+				     packet = Packet},
 		    stop;
 		_ ->
 		    ok
@@ -142,7 +146,7 @@ check_event(From, To, Packet) ->
 					    {xmlelement, "offline", [], []}]}]
 					}),
 			    true
-			end;
+		    end;
 		_ ->
 		    false
 	    end
