@@ -1,35 +1,65 @@
 %%%----------------------------------------------------------------------
 %%% File    : mod_register.erl
 %%% Author  : Alexey Shchepin <alexey@sevcom.net>
-%%% Purpose : 
+%%% Purpose : Inband registration support
 %%% Created :  8 Dec 2002 by Alexey Shchepin <alexey@sevcom.net>
-%%% Id      : $Id: mod_register.erl 307 2005-04-17 18:08:34Z tmallard $
+%%% Id      : $Id: mod_register.erl 375 2005-07-14 22:28:21Z alexey $
 %%%----------------------------------------------------------------------
 
 -module(mod_register).
 -author('alexey@sevcom.net').
--vsn('$Revision$ ').
+-vsn('$Revision: 375 $ ').
 
 -behaviour(gen_mod).
 
--export([start/1, stop/0, process_iq/3]).
+-export([start/2,
+	 stop/1,
+	 stream_feature_register/1,
+	 unauthenticated_iq_register/3,
+	 process_iq/3]).
 
 -include("ejabberd.hrl").
 -include("jlib.hrl").
 
-start(Opts) ->
+start(Host, Opts) ->
     IQDisc = gen_mod:get_opt(iqdisc, Opts, one_queue),
-    gen_iq_handler:add_iq_handler(ejabberd_local, ?NS_REGISTER,
+    gen_iq_handler:add_iq_handler(ejabberd_local, Host, ?NS_REGISTER,
 				  ?MODULE, process_iq, IQDisc),
-    gen_iq_handler:add_iq_handler(ejabberd_sm, ?NS_REGISTER,
+    gen_iq_handler:add_iq_handler(ejabberd_sm, Host, ?NS_REGISTER,
 				  ?MODULE, process_iq, IQDisc),
+    ejabberd_hooks:add(c2s_stream_features, Host,
+ 		       ?MODULE, stream_feature_register, 50),
+    ejabberd_hooks:add(c2s_unauthenticated_iq, Host,
+ 		       ?MODULE, unauthenticated_iq_register, 50),
     ok.
 
-stop() ->
-    gen_iq_handler:remove_iq_handler(ejabberd_local, ?NS_REGISTER),
-    gen_iq_handler:remove_iq_handler(ejabberd_sm, ?NS_REGISTER).
+stop(Host) ->
+    ejabberd_hooks:delete(c2s_stream_features, Host,
+ 			  ?MODULE, stream_feature_register, 50),
+    ejabberd_hooks:delete(c2s_unauthenticated_iq, Host,
+			  ?MODULE, unauthenticated_iq_register, 50),
+    gen_iq_handler:remove_iq_handler(ejabberd_local, Host, ?NS_REGISTER),
+    gen_iq_handler:remove_iq_handler(ejabberd_sm, Host, ?NS_REGISTER).
 
-process_iq(From, To, #iq{type = Type, lang = Lang, sub_el = SubEl} = IQ) ->
+
+stream_feature_register(Acc) ->
+    [{xmlelement, "register",
+      [{"xmlns", ?NS_FEATURE_IQREGISTER}], []} | Acc].
+
+unauthenticated_iq_register(_Acc, Server, #iq{xmlns = ?NS_REGISTER} = IQ) ->
+    ResIQ = process_iq(jlib:make_jid("", "", ""),
+ 		       jlib:make_jid("", Server, ""),
+ 		       IQ),
+    Res1 = jlib:replace_from_to(jlib:make_jid("", Server, ""),
+ 				jlib:make_jid("", "", ""),
+ 				jlib:iq_to_xml(ResIQ)),
+    jlib:remove_attr("to", Res1);
+
+unauthenticated_iq_register(Acc, _Server, _IQ) ->
+    Acc.
+
+process_iq(From, To,
+	   #iq{type = Type, lang = Lang, sub_el = SubEl} = IQ) ->
     case Type of
 	set ->
 	    UTag = xml:get_subtag(SubEl, "username"),
@@ -123,8 +153,8 @@ try_register(User, Server, Password) ->
 	    {error, ?ERR_BAD_REQUEST};
 	_ ->
 	    JID = jlib:make_jid(User, Server, ""),
-	    Access = gen_mod:get_module_opt(?MODULE, access, all),
-	    case acl:match_rule(Access, JID) of
+	    Access = gen_mod:get_module_opt(Server, ?MODULE, access, all),
+	    case acl:match_rule(Server, Access, JID) of
 		deny ->
 		    {error, ?ERR_CONFLICT};
 		allow ->
@@ -147,12 +177,13 @@ try_register(User, Server, Password) ->
 
 
 send_welcome_message(JID) ->
-    case ejabberd_config:get_local_option(welcome_message) of
+    Host = JID#jid.lserver,
+    case ejabberd_config:get_local_option({welcome_message, Host}) of
 	{"", ""} ->
 	    ok;
 	{Subj, Body} ->
 	    ejabberd_router:route(
-	      jlib:make_jid("", ?MYNAME, ""),
+	      jlib:make_jid("", Host, ""),
 	      JID,
 	      {xmlelement, "message", [{"type", "normal"}],
 	       [{xmlelement, "subject", [], [{xmlcdata, Subj}]},
@@ -162,7 +193,8 @@ send_welcome_message(JID) ->
     end.
 
 send_registration_notifications(UJID) ->
-    case ejabberd_config:get_local_option(registration_watchers) of
+    Host = UJID#jid.lserver,
+    case ejabberd_config:get_local_option({registration_watchers, Host}) of
 	[] -> ok;
 	JIDs when is_list(JIDs) ->
 	    Body = lists:flatten(
@@ -175,7 +207,7 @@ send_registration_notifications(UJID) ->
 			  error -> ok;
 			  JID ->
 			      ejabberd_router:route(
-				jlib:make_jid("", ?MYNAME, ""),
+				jlib:make_jid("", Host, ""),
 				JID,
 				{xmlelement, "message", [{"type", "chat"}],
 				 [{xmlelement, "body", [],
