@@ -3,24 +3,25 @@
 %%% Author  : Alexey Shchepin <alexey@sevcom.net>
 %%% Purpose : Roster management
 %%% Created : 15 Dec 2004 by Alexey Shchepin <alexey@sevcom.net>
-%%% Id      : $Id: mod_roster_odbc.erl 307 2005-04-17 18:08:34Z tmallard $
+%%% Id      : $Id: mod_roster_odbc.erl 459 2005-12-11 19:48:31Z alexey $
 %%%----------------------------------------------------------------------
 
 -module(mod_roster_odbc).
 -author('alexey@sevcom.net').
--vsn('$Revision$ ').
+-vsn('$Revision: 459 $ ').
 
 -behaviour(gen_mod).
 
--export([start/1, stop/0,
+-export([start/2, stop/1,
 	 process_iq/3,
 	 process_local_iq/3,
-	 get_subscription_lists/2,
-	 in_subscription/4,
-	 out_subscription/3,
-	 set_items/2,
-	 remove_user/1,
-	 get_jid_info/3]).
+	 get_user_roster/2,
+	 get_subscription_lists/3,
+	 in_subscription/5,
+	 out_subscription/4,
+	 set_items/3,
+	 remove_user/2,
+	 get_jid_info/4]).
 
 -include("ejabberd.hrl").
 -include("jlib.hrl").
@@ -29,37 +30,41 @@
 		 jid,
 		 name = "",
 		 subscription = none,
-		 ask = none}).
+		 ask = none,
+		 groups = []
+		}).
 
-start(Opts) ->
+start(Host, Opts) ->
     IQDisc = gen_mod:get_opt(iqdisc, Opts, one_queue),
-    ejabberd_hooks:add(roster_out_subscription,
-		       ?MODULE, out_subscription, 50),
-    ejabberd_hooks:add(roster_in_subscription,
+    ejabberd_hooks:add(roster_get, Host,
+		       ?MODULE, get_user_roster, 50),
+    ejabberd_hooks:add(roster_in_subscription, Host,
 		       ?MODULE, in_subscription, 50),
-    ejabberd_hooks:add(roster_out_subscription,
+    ejabberd_hooks:add(roster_out_subscription, Host,
 		       ?MODULE, out_subscription, 50),
-    ejabberd_hooks:add(roster_get_subscription_lists,
+    ejabberd_hooks:add(roster_get_subscription_lists, Host,
 		       ?MODULE, get_subscription_lists, 50),
-    ejabberd_hooks:add(roster_get_jid_info,
+    ejabberd_hooks:add(roster_get_jid_info, Host,
 		       ?MODULE, get_jid_info, 50),
-    ejabberd_hooks:add(remove_user,
+    ejabberd_hooks:add(remove_user, Host,
 		       ?MODULE, remove_user, 50),
-    gen_iq_handler:add_iq_handler(ejabberd_sm, ?NS_ROSTER,
+    gen_iq_handler:add_iq_handler(ejabberd_sm, Host, ?NS_ROSTER,
 				  ?MODULE, process_iq, IQDisc).
 
-stop() ->
-    ejabberd_hooks:delete(roster_in_subscription,
+stop(Host) ->
+    ejabberd_hooks:delete(roster_get, Host,
+			  ?MODULE, get_user_roster, 50),
+    ejabberd_hooks:delete(roster_in_subscription, Host,
 			  ?MODULE, in_subscription, 50),
-    ejabberd_hooks:delete(roster_out_subscription,
+    ejabberd_hooks:delete(roster_out_subscription, Host,
 			  ?MODULE, out_subscription, 50),
-    ejabberd_hooks:delete(roster_get_subscription_lists,
+    ejabberd_hooks:delete(roster_get_subscription_lists, Host,
 			  ?MODULE, get_subscription_lists, 50),
-    ejabberd_hooks:delete(roster_get_jid_info,
+    ejabberd_hooks:delete(roster_get_jid_info, Host,
 			  ?MODULE, get_jid_info, 50),
-    ejabberd_hooks:delete(remove_user,
+    ejabberd_hooks:delete(remove_user, Host,
 			  ?MODULE, remove_user, 50),
-    gen_iq_handler:remove_iq_handler(ejabberd_sm, ?NS_ROSTER).
+    gen_iq_handler:remove_iq_handler(ejabberd_sm, Host, ?NS_ROSTER).
 
 
 -define(PSI_ROSTER_WORKAROUND, true).
@@ -105,21 +110,40 @@ process_local_iq(From, To, #iq{type = Type} = IQ) ->
 
 process_iq_get(From, _To, #iq{sub_el = SubEl} = IQ) ->
     LUser = From#jid.luser,
+    LServer = From#jid.lserver,
     Username = ejabberd_odbc:escape(LUser),
     case catch ejabberd_odbc:sql_query(
+		 LServer,
 		 ["select username, jid, nick, subscription, ask, "
 		  "server, subscribe, type from rosterusers "
 		  "where username='", Username, "'"]) of
 	{selected, ["username", "jid", "nick", "subscription", "ask",
 		    "server", "subscribe", "type"],
 	 Items} when is_list(Items) ->
+	    JIDGroups = case catch ejabberd_odbc:sql_query(
+				     LServer,
+				     ["select jid, grp from rostergroups "
+				      "where username='", Username, "'"]) of
+			    {selected, ["jid","grp"],
+			     JGrps} when is_list(JGrps) ->
+				JGrps;
+			    _ ->
+				[]
+			end,
 	    XItems = lists:flatmap(
 		       fun(I) ->
 			       case raw_to_record(I) of
 				   error ->
 				       [];
 				   R ->
-				       [item_to_xml(R)]
+				       SJID = jlib:jid_to_string(R#roster.jid),
+				       Groups = lists:flatmap(
+						  fun({S, G}) when S == SJID ->
+							  [G];
+						     (_) ->
+							  []
+						  end, JIDGroups),
+				       [item_to_xml(R#roster{groups = Groups})]
 			       end
 		       end, Items),
 	    IQ#iq{type = result,
@@ -130,6 +154,31 @@ process_iq_get(From, _To, #iq{sub_el = SubEl} = IQ) ->
 	    IQ#iq{type = error, sub_el = [SubEl, ?ERR_INTERNAL_SERVER_ERROR]}
     end.
 
+get_user_roster(Acc, {LUser, LServer}) ->
+    Username = ejabberd_odbc:escape(LUser),
+    case catch ejabberd_odbc:sql_query(
+		 LServer,
+		 ["select username, jid, nick, subscription, ask, "
+		  "server, subscribe, type from rosterusers "
+		  "where username='", Username, "'"]) of
+	{selected, ["username", "jid", "nick", "subscription", "ask",
+		    "server", "subscribe", "type"],
+	 Items} when is_list(Items) ->
+	    RItems = lists:flatmap(
+		       fun(I) ->
+			       case raw_to_record(I) of
+				   error ->
+				       [];
+				   R ->
+				       [R]
+			       end
+		       end, Items),
+	    RItems ++ Acc;
+	_ ->
+	    Acc
+    end.
+
+
 item_to_xml(Item) ->
     Attrs1 = [{"jid", jlib:jid_to_string(Item#roster.jid)}],
     Attrs2 = case Item#roster.name of
@@ -158,11 +207,9 @@ item_to_xml(Item) ->
 		_ ->
 		    Attrs3
 	    end,
-    % TODO
-    %SubEls = lists:map(fun(G) ->
-    %    		       {xmlelement, "group", [], [{xmlcdata, G}]}
-    %    	       end, Item#roster.groups),
-    SubEls = [],
+    SubEls = lists:map(fun(G) ->
+        		       {xmlelement, "group", [], [{xmlcdata, G}]}
+        	       end, Item#roster.groups),
     {xmlelement, "item", Attrs, SubEls}.
 
 
@@ -173,7 +220,7 @@ process_iq_set(From, To, #iq{sub_el = SubEl} = IQ) ->
 
 process_item_set(From, To, {xmlelement, _Name, Attrs, Els}) ->
     JID1 = jlib:string_to_jid(xml:get_attr_s("jid", Attrs)),
-    #jid{user = User, luser = LUser} = From,
+    #jid{user = User, luser = LUser, lserver = LServer} = From,
     case JID1 of
 	error ->
 	    ok;
@@ -182,64 +229,69 @@ process_item_set(From, To, {xmlelement, _Name, Attrs, Els}) ->
 	    LJID = jlib:jid_tolower(JID1),
 	    Username = ejabberd_odbc:escape(LUser),
 	    SJID = ejabberd_odbc:escape(jlib:jid_to_string(LJID)),
-	    case catch ejabberd_odbc:sql_query(
-			 ["select username, jid, nick, subscription, ask, "
-			  "server, subscribe, type from rosterusers "
-			  "where username='", Username, "' "
-			  "and jid='", SJID, "'"]) of
-		{selected, ["username", "jid", "nick", "subscription", "ask",
-			    "server", "subscribe", "type"],
-		 Res} ->
-		    Item = case Res of
-			       [] ->
-				   #roster{user = LUser,
-					   jid = LJID};
-			       [I] ->
-				   (raw_to_record(I))#roster{user = LUser,
-							     jid = JID,
-							     name = ""}
-			   end,
-		    Item1 = process_item_attrs(Item, Attrs),
-		    case Item1#roster.subscription of
+	    F = fun() ->
+			{selected,
+			 ["username", "jid", "nick", "subscription",
+			  "ask", "server", "subscribe", "type"],
+			 Res} =
+			    ejabberd_odbc:sql_query_t(
+			      ["select username, jid, nick, subscription, "
+			       "ask, server, subscribe, type from rosterusers "
+			       "where username='", Username, "' "
+			       "and jid='", SJID, "'"]),
+			Item = case Res of
+				   [] ->
+				       #roster{user = LUser,
+					       jid = LJID};
+				   [I] ->
+				       (raw_to_record(I))#roster{user = LUser,
+								 jid = JID,
+								 name = ""}
+			       end,
+			Item1 = process_item_attrs(Item, Attrs),
+			Item2 = process_item_els(Item1, Els),
+			case Item2#roster.subscription of
+			    remove ->
+				ejabberd_odbc:sql_query_t(
+				  ["delete from rosterusers "
+				   "      where username='", Username, "' "
+				   "        and jid='", SJID, "';"
+				   "delete from rostergroups "
+				   "      where username='", Username, "' "
+				   "        and jid='", SJID, "'"]);
+			    _ ->
+				ItemVals = record_to_string(Item2),
+				ItemGroups = groups_to_string(Item2),
+				ejabberd_odbc:sql_query_t(
+				  ["delete from rosterusers "
+				   "      where username='", Username, "' "
+				   "        and jid='", SJID, "';"
+				   "insert into rosterusers("
+				   "              username, jid, nick, "
+				   "              subscription, ask, "
+				   "              server, subscribe, type) "
+				   " values ", ItemVals, ";"
+				   "delete from rostergroups "
+				   "      where username='", Username, "' "
+				   "        and jid='", SJID, "';",
+				   [["insert into rostergroups("
+				     "              username, jid, grp) "
+				     " values ", ItemGroup, ";"] ||
+				       ItemGroup <- ItemGroups]])
+			end,
+			{Item, Item2}
+		end,
+	    case ejabberd_odbc:sql_transaction(LServer, F) of
+		{atomic, {OldItem, Item}} ->
+		    push_item(User, LServer, To, Item),
+		    case Item#roster.subscription of
 			remove ->
-			    catch ejabberd_odbc:sql_query(
-				    ["begin;"
-				     "delete from rosterusers "
-				     "      where username='", Username, "' "
-				     "        and jid='", SJID, "';"
-				     "delete from rostergroups "
-				     "      where username='", Username, "' "
-				     "        and jid='", SJID, "';"
-				     "commit"]);
-			_ ->
-			    ItemVals = record_to_string(Item1),
-			    catch ejabberd_odbc:sql_query(
-				    ["begin;"
-				     "delete from rosterusers "
-				     "      where username='", Username, "' "
-				     "        and jid='", SJID, "';"
-				     "insert into rosterusers("
-				     "              username, jid, nick, "
-				     "              subscription, ask, "
-				     "              server, subscribe, type) "
-				     " values ", ItemVals, ";"
-				     "delete from rostergroups "
-				     "      where username='", Username, "' "
-				     "        and jid='", SJID, "';"
-				     % TODO
-				     "commit"])
-
-			    %mnesia:write(Item1)
-		    end,
-		    push_item(User, To, Item1),
-		    case Item1#roster.subscription of
-			remove ->
-			    IsTo = case Item#roster.subscription of
+			    IsTo = case OldItem#roster.subscription of
 				       both -> true;
 				       to -> true;
 				       _ -> false
 				   end,
-			    IsFrom = case Item#roster.subscription of
+			    IsFrom = case OldItem#roster.subscription of
 					 both -> true;
 					 from -> true;
 					 _ -> false
@@ -247,7 +299,7 @@ process_item_set(From, To, {xmlelement, _Name, Attrs, Els}) ->
 			    if IsTo ->
 				    ejabberd_router:route(
 				      jlib:jid_remove_resource(From),
-				      jlib:make_jid(Item#roster.jid),
+				      jlib:make_jid(OldItem#roster.jid),
 				      {xmlelement, "presence",
 				       [{"type", "unsubscribe"}],
 				       []});
@@ -256,7 +308,7 @@ process_item_set(From, To, {xmlelement, _Name, Attrs, Els}) ->
 			    if IsFrom ->
 				    ejabberd_router:route(
 				      jlib:jid_remove_resource(From),
-				      jlib:make_jid(Item#roster.jid),
+				      jlib:make_jid(OldItem#roster.jid),
 				      {xmlelement, "presence",
 				       [{"type", "unsubscribed"}],
 				       []});
@@ -303,69 +355,66 @@ process_item_attrs(Item, []) ->
     Item.
 
 
-% TODO
-%process_item_els(Item, [{xmlelement, Name, Attrs, SEls} | Els]) ->
-%    case Name of
-%	"group" ->
-%	    Groups = [xml:get_cdata(SEls) | Item#roster.groups],
-%	    process_item_els(Item#roster{groups = Groups}, Els);
-%	_ ->
-%	    case xml:get_attr_s("xmlns", Attrs) of
-%		"" ->
-%		    process_item_els(Item, Els);
-%		_ ->
-%		    XEls = [{xmlelement, Name, Attrs, SEls} | Item#roster.xs],
-%		    process_item_els(Item#roster{xs = XEls}, Els)
-%	    end
-%    end;
+process_item_els(Item, [{xmlelement, Name, Attrs, SEls} | Els]) ->
+    case Name of
+	"group" ->
+	    Groups = [xml:get_cdata(SEls) | Item#roster.groups],
+	    process_item_els(Item#roster{groups = Groups}, Els);
+	_ ->
+	    process_item_els(Item, Els)
+    end;
 process_item_els(Item, [{xmlcdata, _} | Els]) ->
     process_item_els(Item, Els);
 process_item_els(Item, []) ->
     Item.
 
 
-push_item(User, From, Item) ->
+push_item(User, Server, From, Item) ->
     ejabberd_sm:route(jlib:make_jid("", "", ""),
-		      jlib:make_jid(User, "", ""),
+		      jlib:make_jid(User, Server, ""),
 		      {xmlelement, "broadcast", [],
 		       [{item,
 			 Item#roster.jid,
 			 Item#roster.subscription}]}),
     lists:foreach(fun(Resource) ->
-			  push_item(User, Resource, From, Item)
-		  end, ejabberd_sm:get_user_resources(User, 'TODO')).
+			  push_item(User, Server, Resource, From, Item)
+		  end, ejabberd_sm:get_user_resources(User, Server)).
 
 % TODO: don't push to those who not load roster
 -ifdef(PSI_ROSTER_WORKAROUND).
 
-push_item(User, Resource, _From, Item) ->
+push_item(User, Server, Resource, _From, Item) ->
     ResIQ = #iq{type = set, xmlns = ?NS_ROSTER,
+		id = "push",
 		sub_el = [{xmlelement, "query",
 			   [{"xmlns", ?NS_ROSTER}],
 			   [item_to_xml(Item)]}]},
     ejabberd_router:route(
-      jlib:make_jid(User, ?MYNAME, Resource),
-      jlib:make_jid(User, ?MYNAME, Resource),
+      jlib:make_jid(User, Server, Resource),
+      jlib:make_jid(User, Server, Resource),
       jlib:iq_to_xml(ResIQ)).
 
 -else.
 
 push_item(User, Resource, From, Item) ->
     ResIQ = #iq{type = set, xmlns = ?NS_ROSTER,
+		id = "push",
 		sub_el = [{xmlelement, "query",
 			   [{"xmlns", ?NS_ROSTER}],
 			   [item_to_xml(Item)]}]},
     ejabberd_router:route(
       From,
-      jlib:make_jid(User, ?MYNAME, Resource),
+      jlib:make_jid(User, Server, Resource),
       jlib:iq_to_xml(ResIQ)).
 
 -endif.
 
-get_subscription_lists(_, User) ->
+get_subscription_lists(_, User, Server) ->
     LUser = jlib:nodeprep(User),
+    LServer = jlib:nameprep(Server),
     Username = ejabberd_odbc:escape(LUser),
     case catch ejabberd_odbc:sql_query(
+		 LServer,
 		 ["select username, jid, nick, subscription, ask, "
 		  "server, subscribe, type from rosterusers "
 		  "where username='", Username, "'"]) of
@@ -377,7 +426,8 @@ get_subscription_lists(_, User) ->
 	    {[], []}
     end.
 
-fill_subscription_lists([I | Is], F, T) ->
+fill_subscription_lists([RawI | Is], F, T) ->
+    I = raw_to_record(RawI),
     J = I#roster.jid,
     case I#roster.subscription of
 	both ->
@@ -398,95 +448,124 @@ ask_to_pending(Ask) -> Ask.
 
 
 
-in_subscription(_, User, JID, Type) ->
-    process_subscription(in, User, JID, Type).
+in_subscription(_, User, Server, JID, Type) ->
+    process_subscription(in, User, Server, JID, Type).
 
-out_subscription(User, JID, Type) ->
-    process_subscription(out, User, JID, Type).
+out_subscription(User, Server, JID, Type) ->
+    process_subscription(out, User, Server, JID, Type).
 
-process_subscription(Direction, User, JID1, Type) ->
+process_subscription(Direction, User, Server, JID1, Type) ->
     LUser = jlib:nodeprep(User),
+    LServer = jlib:nameprep(Server),
     LJID = jlib:jid_tolower(JID1),
     Username = ejabberd_odbc:escape(LUser),
     SJID = ejabberd_odbc:escape(jlib:jid_to_string(LJID)),
-    Item = case catch ejabberd_odbc:sql_query(
-			["select username, jid, nick, subscription, ask, "
-			 "server, subscribe, type from rosterusers "
-			 "where username='", Username, "' "
-			 "and jid='", SJID, "'"]) of
-	       {selected, ["username", "jid", "nick", "subscription", "ask",
-			   "server", "subscribe", "type"],
-		[I]} ->
-		   raw_to_record(I);
-	       _ ->
-		   #roster{user = LUser,
-			   jid = LJID}
-	   end,
-    NewState = case Direction of
-		   out ->
-		       out_state_change(Item#roster.subscription,
-					Item#roster.ask,
-					Type);
-		   in ->
-		       in_state_change(Item#roster.subscription,
-				       Item#roster.ask,
-				       Type)
-	       end,
-    AutoReply = case Direction of
-		    out ->
-			none;
-		    in ->
-			in_auto_reply(Item#roster.subscription,
-				      Item#roster.ask,
-				      Type)
-		end,
-    Push = case NewState of
-	       none ->
-		   none;
-	       {Subscription, Pending} ->
-		   NewItem = Item#roster{subscription = Subscription,
-					 ask = Pending},
-		   ItemVals = record_to_string(NewItem),
-		   catch ejabberd_odbc:sql_query(
-			   ["begin;"
-			    "delete from rosterusers "
-			    "      where username='", Username, "' "
-			    "        and jid='", SJID, "';"
-			    "insert into rosterusers("
-			    "              username, jid, nick, "
-			    "              subscription, ask, "
-			    "              server, subscribe, type) "
-			    " values ", ItemVals, ";"
-			    "commit"]),
-		   {push, NewItem}
-	   end,
-    case AutoReply of
-	none ->
-	    ok;
+    F = fun() ->
+		Item =
+		    case ejabberd_odbc:sql_query_t(
+			   ["select username, jid, nick, subscription, ask, "
+			    "server, subscribe, type from rosterusers "
+			    "where username='", Username, "' "
+			    "and jid='", SJID, "'"]) of
+			{selected,
+			 ["username", "jid", "nick", "subscription", "ask",
+			  "server", "subscribe", "type"],
+			 [I]} ->
+			    R = raw_to_record(I),
+			    Groups =
+				case ejabberd_odbc:sql_query_t(
+				       ["select grp from rostergroups "
+					"where username='", Username, "' "
+					"and jid='", SJID, "'"]) of
+				    {selected, ["grp"], JGrps} when is_list(JGrps) ->
+					[JGrp || {JGrp} <- JGrps];
+				    _ ->
+					[]
+				end,
+			    R#roster{groups = Groups};
+			{selected,
+			 ["username", "jid", "nick", "subscription", "ask",
+			  "server", "subscribe", "type"],
+			 []} ->
+			    #roster{user = LUser,
+				    jid = LJID}
+		    end,
+		NewState = case Direction of
+			       out ->
+				   out_state_change(Item#roster.subscription,
+						    Item#roster.ask,
+						    Type);
+			       in ->
+				   in_state_change(Item#roster.subscription,
+						   Item#roster.ask,
+						   Type)
+			   end,
+		AutoReply = case Direction of
+				out ->
+				    none;
+				in ->
+				    in_auto_reply(Item#roster.subscription,
+						  Item#roster.ask,
+						  Type)
+			    end,
+		case NewState of
+		    none ->
+			{none, AutoReply};
+		    {Subscription, Pending} ->
+			NewItem = Item#roster{subscription = Subscription,
+					      ask = Pending},
+			ItemVals = record_to_string(NewItem),
+			ejabberd_odbc:sql_query_t(
+			  ["delete from rosterusers "
+			   "      where username='", Username, "' "
+			   "        and jid='", SJID, "';"
+			   "insert into rosterusers("
+			   "              username, jid, nick, "
+			   "              subscription, ask, "
+			   "              server, subscribe, type) "
+			   " values ", ItemVals]),
+			{{push, NewItem}, AutoReply}
+		end
+	end,
+    case ejabberd_odbc:sql_transaction(LServer, F) of
+	{atomic, {Push, AutoReply}} ->
+	    case AutoReply of
+		none ->
+		    ok;
+		_ ->
+		    T = case AutoReply of
+			    subscribed -> "subscribed";
+			    unsubscribed -> "unsubscribed"
+			end,
+		    ejabberd_router:route(
+		      jlib:make_jid(User, Server, ""), JID1,
+		      {xmlelement, "presence", [{"type", T}], []})
+	    end,
+	    case Push of
+		{push, Item} ->
+		    push_item(User, Server,
+			      jlib:make_jid("", Server, ""), Item),
+		    true;
+		none ->
+		    false
+	    end;
 	_ ->
-	    T = case AutoReply of
-		    subscribed -> "subscribed";
-		    unsubscribed -> "unsubscribed"
-		end,
-	    ejabberd_router:route(
-	      jlib:make_jid(User, ?MYNAME, ""), JID1,
-	      {xmlelement, "presence", [{"type", T}], []})
-    end,
-    case Push of
-	{push, PushItem} ->
-	    push_item(User, jlib:make_jid("", ?MYNAME, ""), PushItem),
-	    true;
-	none ->
 	    false
     end.
 
 
 %% in_state_change(Subscription, Pending, Type) -> NewState
 %% NewState = none | {NewSubscription, NewPending}
+-ifdef(ROSTER_GATEWAY_WORKAROUND).
+-define(NNSD, {to, none}).
+-define(NISD, {to, in}).
+-else.
+-define(NNSD, none).
+-define(NISD, none).
+-endif.
 
 in_state_change(none, none, subscribe)    -> {none, in};
-in_state_change(none, none, subscribed)   -> {to, none}; % Workaround for gateways
-%in_state_change(none, none, subscribed)   -> none;
+in_state_change(none, none, subscribed)   -> ?NNSD;
 in_state_change(none, none, unsubscribe)  -> none;
 in_state_change(none, none, unsubscribed) -> none;
 in_state_change(none, out,  subscribe)    -> {none, both};
@@ -494,8 +573,7 @@ in_state_change(none, out,  subscribed)   -> {to, none};
 in_state_change(none, out,  unsubscribe)  -> none;
 in_state_change(none, out,  unsubscribed) -> {none, none};
 in_state_change(none, in,   subscribe)    -> none;
-in_state_change(none, in,   subscribed)   -> {to, in}; % Workaround for gateways
-%in_state_change(none, in,   subscribed)   -> none;
+in_state_change(none, in,   subscribed)   -> ?NISD;
 in_state_change(none, in,   unsubscribe)  -> {none, none};
 in_state_change(none, in,   unsubscribed) -> none;
 in_state_change(none, both, subscribe)    -> none;
@@ -572,49 +650,79 @@ in_auto_reply(both, none, unsubscribe)  -> unsubscribed;
 in_auto_reply(_,    _,    _)  ->           none.
 
 
-remove_user(User) ->
+remove_user(User, Server) ->
     LUser = jlib:nodeprep(User),
+    LServer = jlib:nameprep(Server),
     Username = ejabberd_odbc:escape(LUser),
-    catch ejabberd_odbc:sql_query(
-	    ["begin;"
-	     "delete from rosterusers "
-	     "      where username='", Username, "';"
-	     "delete from rostergroups "
-	     "      where username='", Username, "';"
-	     "commit"]),
+    ejabberd_odbc:sql_transaction(
+      LServer,
+      fun() ->
+	      ejabberd_odbc:sql_query_t(
+		["delete from rosterusers "
+		 "      where username='", Username, "';"
+		 "delete from rostergroups "
+		 "      where username='", Username, "'"])
+      end),
     ok.
 
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
-set_items(User, SubEl) ->
+set_items(User, Server, SubEl) ->
     {xmlelement, _Name, _Attrs, Els} = SubEl,
     LUser = jlib:nodeprep(User),
-    F = fun() ->
-		lists:foreach(fun(El) -> process_item_set_t(LUser, El) end, Els)
-	end,
-    mnesia:transaction(F).
+    LServer = jlib:nameprep(Server),
+    catch ejabberd_odbc:sql_query(
+	    LServer,
+	    ["begin;",
+	     lists:map(fun(El) ->
+			       process_item_set_t(LUser, LServer, El)
+		       end, Els),
+	     "commit"]).
 
-process_item_set_t(LUser, {xmlelement, _Name, Attrs, Els}) ->
+process_item_set_t(LUser, LServer, {xmlelement, _Name, Attrs, Els}) ->
     JID1 = jlib:string_to_jid(xml:get_attr_s("jid", Attrs)),
     case JID1 of
 	error ->
-	    ok;
+	    [];
 	_ ->
 	    JID = {JID1#jid.user, JID1#jid.server, JID1#jid.resource},
 	    LJID = {JID1#jid.luser, JID1#jid.lserver, JID1#jid.lresource},
+	    Username = ejabberd_odbc:escape(LUser),
+	    SJID = ejabberd_odbc:escape(jlib:jid_to_string(LJID)),
 	    Item = #roster{user = LUser,
 			   jid = LJID},
 	    Item1 = process_item_attrs_ws(Item, Attrs),
 	    Item2 = process_item_els(Item1, Els),
 	    case Item2#roster.subscription of
-		remove ->
-		    mnesia:delete({roster, {LUser, LJID}});
-		_ ->
-		    mnesia:write(Item2)
+	        remove ->
+		    ["delete from rosterusers "
+		     "      where username='", Username, "' "
+		     "        and jid='", SJID, "';"
+		     "delete from rostergroups "
+		     "      where username='", Username, "' "
+		     "        and jid='", SJID, "';"];
+	        _ ->
+	            ItemVals = record_to_string(Item1),
+		    ItemGroups = groups_to_string(Item2),
+		    ["delete from rosterusers "
+		     "      where username='", Username, "' "
+		     "        and jid='", SJID, "';"
+		     "insert into rosterusers("
+		     "              username, jid, nick, "
+		     "              subscription, ask, "
+		     "              server, subscribe, type) "
+		     " values ", ItemVals, ";"
+		     "delete from rostergroups "
+		     "      where username='", Username, "' "
+		     "        and jid='", SJID, "';",
+		     [["insert into rostergroups("
+		       "              username, jid, grp) "
+		       " values ", ItemGroup, ";"] ||
+			 ItemGroup <- ItemGroups]]
 	    end
     end;
-process_item_set_t(_LUser, _) ->
-    ok.
+process_item_set_t(_LUser, _LServer, _) ->
+    [].
 
 process_item_attrs_ws(Item, [{Attr, Val} | Attrs]) ->
     case Attr of
@@ -659,29 +767,70 @@ process_item_attrs_ws(Item, []) ->
 
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
-get_jid_info(_, User, JID) ->
-% TODO
-%    LUser = jlib:nodeprep(User),
-%    LJID = jlib:jid_tolower(JID),
-%    case catch mnesia:dirty_read(roster, {LUser, LJID}) of
-%	[#roster{subscription = Subscription, groups = Groups}] ->
-%	    {Subscription, Groups};
-%	_ ->
-%	    LRJID = jlib:jid_tolower(jlib:jid_remove_resource(JID)),
-%	    if
-%		LRJID == LJID ->
-%		    {none, []};
-%		true ->
-%		    case catch mnesia:dirty_read(roster, {LUser, LRJID}) of
-%			[#roster{subscription = Subscription,
-%				 groups = Groups}] ->
-%			    {Subscription, Groups};
-%			_ ->
-%			    {none, []}
-%		    end
-%	    end
-%    end.
-{none, []}.
+get_jid_info(_, User, Server, JID) ->
+    LUser = jlib:nodeprep(User),
+    LServer = jlib:nameprep(Server),
+    LJID = jlib:jid_tolower(JID),
+    Username = ejabberd_odbc:escape(LUser),
+    SJID = ejabberd_odbc:escape(jlib:jid_to_string(LJID)),
+    case catch ejabberd_odbc:sql_query(
+		 LServer,
+		 ["select subscription from rosterusers "
+		  "where username='", Username, "' "
+		  "and jid='", SJID, "'"]) of
+	{selected, ["subscription"], [{SSubscription}]} ->
+	    Subscription = case SSubscription of
+			       "B" -> both;
+			       "T" -> to;
+			       "F" -> from;
+			       _ -> none
+			   end,
+	    Groups = case catch ejabberd_odbc:sql_query(
+				  LServer,
+				  ["select grp from rostergroups "
+				   "where username='", Username, "' "
+				   "and jid='", SJID, "'"]) of
+			 {selected, ["grp"], JGrps} when is_list(JGrps) ->
+			     [JGrp || {JGrp} <- JGrps];
+			 _ ->
+			     []
+		     end,
+	    {Subscription, Groups};
+	_ ->
+	    LRJID = jlib:jid_tolower(jlib:jid_remove_resource(JID)),
+	    if
+		LRJID == LJID ->
+		    {none, []};
+		true ->
+		    SRJID = ejabberd_odbc:escape(jlib:jid_to_string(LRJID)),
+		    case catch ejabberd_odbc:sql_query(
+				 LServer,
+				 ["select subscription from rosterusers "
+				  "where username='", Username, "' "
+				  "and jid='", SRJID, "'"]) of
+			{selected, ["subscription"], [{SSubscription}]} ->
+			    Subscription = case SSubscription of
+					       "B" -> both;
+					       "T" -> to;
+					       "F" -> from;
+					       _ -> none
+					   end,
+			    Groups = case catch ejabberd_odbc:sql_query(
+						  LServer,
+						  ["select grp from rostergroups "
+						   "where username='", Username, "' "
+						   "and jid='", SRJID, "'"]) of
+					 {selected, ["grp"], JGrps} when is_list(JGrps) ->
+					     [JGrp || {JGrp} <- JGrps];
+					 _ ->
+					     []
+				     end,
+			    {Subscription, Groups};
+			_ ->
+			    {none, []}
+		    end
+	    end
+    end.
 
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
@@ -719,7 +868,7 @@ record_to_string(#roster{user = User,
 			 subscription = Subscription,
 			 ask = Ask}) ->
     Username = ejabberd_odbc:escape(User),
-    SJID = ejabberd_odbc:escape(jlib:jid_to_string(JID)),
+    SJID = ejabberd_odbc:escape(jlib:jid_to_string(jlib:jid_tolower(JID))),
     Nick = ejabberd_odbc:escape(Name),
     SSubscription = case Subscription of
 			both -> "B";
@@ -743,3 +892,13 @@ record_to_string(#roster{user = User,
      "'", SAsk, "',"
      "'N', '', 'item')"].
 
+groups_to_string(#roster{user = User,
+			 jid = JID,
+			 groups = Groups}) ->
+    Username = ejabberd_odbc:escape(User),
+    SJID = ejabberd_odbc:escape(jlib:jid_to_string(jlib:jid_tolower(JID))),
+    [["("
+      "'", Username, "',"
+      "'", SJID, "',"
+      "'", ejabberd_odbc:escape(Group), "')"] || Group <- Groups].
+
