@@ -3,16 +3,17 @@
 %%% Author  : Alexey Shchepin <alexey@sevcom.net>
 %%% Purpose : 
 %%% Created :  2 Jan 2003 by Alexey Shchepin <alexey@sevcom.net>
-%%% Id      : $Id: mod_vcard.erl 307 2005-04-17 18:08:34Z tmallard $
+%%% Id      : $Id: mod_vcard.erl 437 2005-11-19 01:20:05Z alexey $
 %%%----------------------------------------------------------------------
 
 -module(mod_vcard).
 -author('alexey@sevcom.net').
--vsn('$Revision$ ').
+-vsn('$Revision: 437 $ ').
 
 -behaviour(gen_mod).
 
--export([start/1, init/2, stop/0,
+-export([start/2, init/3, stop/1,
+	 get_sm_features/5,
 	 process_local_iq/3,
 	 process_sm_iq/3,
 	 reindex_vcards/0,
@@ -40,8 +41,9 @@
 		      }).
 -record(vcard, {us, vcard}).
 
+-define(PROCNAME, ejabberd_mod_vcard).
 
-start(Opts) ->
+start(Host, Opts) ->
     mnesia:create_table(vcard, [{disc_only_copies, [node()]},
 				{attributes, record_info(fields, vcard)}]),
     mnesia:create_table(vcard_search,
@@ -61,53 +63,71 @@ start(Opts) ->
     mnesia:add_table_index(vcard_search, lorgname),
     mnesia:add_table_index(vcard_search, lorgunit),
 
-    ejabberd_hooks:add(remove_user,
+    ejabberd_hooks:add(remove_user, Host,
 		       ?MODULE, remove_user, 50),
     IQDisc = gen_mod:get_opt(iqdisc, Opts, one_queue),
-    gen_iq_handler:add_iq_handler(ejabberd_local, ?NS_VCARD,
+    gen_iq_handler:add_iq_handler(ejabberd_local, Host, ?NS_VCARD,
 				  ?MODULE, process_local_iq, IQDisc),
-    gen_iq_handler:add_iq_handler(ejabberd_sm, ?NS_VCARD,
+    gen_iq_handler:add_iq_handler(ejabberd_sm, Host, ?NS_VCARD,
 				  ?MODULE, process_sm_iq, IQDisc),
-    catch mod_disco:register_sm_feature(?NS_VCARD),
-    Hosts = gen_mod:get_hosts(Opts, "vjud."),
+    ejabberd_hooks:add(disco_sm_features, Host, ?MODULE, get_sm_features, 50),
+    MyHost = gen_mod:get_opt(host, Opts, "vjud." ++ Host),
     Search = gen_mod:get_opt(search, Opts, true),
-    register(ejabberd_mod_vcard, spawn(?MODULE, init, [Hosts, Search])).
+    register(gen_mod:get_module_proc(Host, ?PROCNAME),
+	     spawn(?MODULE, init, [MyHost, Host, Search])).
 
 
-init(Hosts, Search) ->
+init(Host, ServerHost, Search) ->
     case Search of
 	false ->
-	    loop(Hosts);
+	    loop(Host, ServerHost);
 	_ ->
-	    ejabberd_router:register_routes(Hosts),
-	    loop(Hosts)
+	    ejabberd_router:register_route(Host),
+	    loop(Host, ServerHost)
     end.
 
-loop(Hosts) ->
+loop(Host, ServerHost) ->
     receive
 	{route, From, To, Packet} ->
-	    case catch do_route(From, To, Packet) of
+	    case catch do_route(ServerHost, From, To, Packet) of
 		{'EXIT', Reason} ->
 		    ?ERROR_MSG("~p", [Reason]);
 		_ ->
 		    ok
 	    end,
-	    loop(Hosts);
+	    loop(Host, ServerHost);
 	stop ->
-	    catch ejabberd_router:unregister_routes(Hosts),
+	    ejabberd_router:unregister_route(Host),
 	    ok;
 	_ ->
-	    loop(Hosts)
+	    loop(Host, ServerHost)
     end.
 
-stop() ->
-    ejabberd_hooks:delete(remove_user,
+stop(Host) ->
+    ejabberd_hooks:delete(remove_user, Host,
 			  ?MODULE, remove_user, 50),
-    gen_iq_handler:remove_iq_handler(ejabberd_local, ?NS_VCARD),
-    gen_iq_handler:remove_iq_handler(ejabberd_sm, ?NS_VCARD),
-    catch mod_disco:unregister_sm_feature(?NS_VCARD),
-    ejabberd_mod_vcard ! stop,
-    ok.
+    gen_iq_handler:remove_iq_handler(ejabberd_local, Host, ?NS_VCARD),
+    gen_iq_handler:remove_iq_handler(ejabberd_sm, Host, ?NS_VCARD),
+    ejabberd_hooks:delete(disco_sm_features, Host, ?MODULE, get_sm_features, 50),
+    Proc = gen_mod:get_module_proc(Host, ?PROCNAME),
+    Proc ! stop,
+    {wait, Proc}.
+
+get_sm_features({error, _Error} = Acc, _From, _To, _Node, _Lang) ->
+    Acc;
+ 
+get_sm_features(Acc, _From, _To, Node, _Lang) ->
+    case Node of
+	[] ->
+	    case Acc of
+		{result, Features} ->
+		    {result, [?NS_VCARD | Features]};
+		empty ->
+		    {result, [?NS_VCARD]}
+	    end;
+ 	_ ->
+	    Acc
+     end.
 
 process_local_iq(_From, _To, #iq{type = Type, lang = Lang, sub_el = SubEl} = IQ) ->
     case Type of
@@ -137,7 +157,7 @@ process_local_iq(_From, _To, #iq{type = Type, lang = Lang, sub_el = SubEl} = IQ)
 process_sm_iq(From, To, #iq{type = Type, sub_el = SubEl} = IQ) ->
     case Type of
 	set ->
-	    #jid{user = User, lserver = LServer, luser = LUser} = From,
+	    #jid{user = User, lserver = LServer} = From,
 	    case lists:member(LServer, ?MYHOSTS) of
 		true ->
 		    set_vcard(User, LServer, SubEl),
@@ -269,7 +289,7 @@ set_vcard(User, LServer, VCARD) ->
 
 
 
-do_route(From, To, Packet) ->
+do_route(ServerHost, From, To, Packet) ->
     #jid{user = User, resource = Resource} = To,
     if
 	(User /= "") or (Resource /= "") ->
@@ -307,7 +327,7 @@ do_route(From, To, Packet) ->
 						    [{xmlelement, "x",
 						      [{"xmlns", ?NS_XDATA},
 						       {"type", "result"}],
-						      search_result(Lang, To, XData)
+						      search_result(Lang, To, ServerHost, XData)
 						     }]}]},
 					    ejabberd_router:route(
 					      To, From, jlib:iq_to_xml(ResIQ))
@@ -324,7 +344,7 @@ do_route(From, To, Packet) ->
 						  From,
 						  jlib:iq_to_xml(ResIQ))
 		    end;
-		#iq{type = Type, xmlns = ?NS_DISCO_INFO, sub_el = SubEl} ->
+		#iq{type = Type, xmlns = ?NS_DISCO_INFO} ->
 		    case Type of
 			set ->
 			    Err = jlib:make_error_reply(
@@ -352,7 +372,7 @@ do_route(From, To, Packet) ->
 						  From,
 						  jlib:iq_to_xml(ResIQ))
 		    end;
-		#iq{type = Type, xmlns = ?NS_DISCO_ITEMS, sub_el = SubEl} ->
+		#iq{type = Type, xmlns = ?NS_DISCO_ITEMS} ->
 		    case Type of
 			set ->
 			    Err = jlib:make_error_reply(
@@ -363,7 +383,7 @@ do_route(From, To, Packet) ->
 				IQ#iq{type = result,
 				      sub_el = [{xmlelement,
 						 "query",
-						 [{"xmlns", ?NS_DISCO_INFO}],
+						 [{"xmlns", ?NS_DISCO_ITEMS}],
 						 []}]},
 			    ejabberd_router:route(To,
 						  From,
@@ -417,7 +437,7 @@ find_xdata_el1([_ | Els]) ->
 	{xmlelement, "field", [{"label", translate:translate(Lang, Label)},
 			       {"var", Var}], []}).
 
-search_result(Lang, JID, Data) ->
+search_result(Lang, JID, ServerHost, Data) ->
     [{xmlelement, "title", [],
       [{xmlcdata, translate:translate(Lang, "Results of search in ") ++
 	jlib:jid_to_string(JID)}]},
@@ -434,7 +454,7 @@ search_result(Lang, JID, Data) ->
        ?LFIELD("email", "email"),
        ?LFIELD("Organization Name", "orgname"),
        ?LFIELD("Organization Unit", "orgunit")
-      ]}] ++ lists:map(fun record_to_item/1, search(JID#jid.lserver, Data)).
+      ]}] ++ lists:map(fun record_to_item/1, search(ServerHost, Data)).
 
 -define(FIELD(Var, Val),
 	{xmlelement, "field", [{"var", Var}],
@@ -463,7 +483,8 @@ record_to_item(R) ->
 
 search(LServer, Data) ->
     MatchSpec = make_matchspec(LServer, Data),
-    AllowReturnAll = gen_mod:get_module_opt(?MODULE, allow_return_all, false),
+    AllowReturnAll = gen_mod:get_module_opt(LServer, ?MODULE,
+					    allow_return_all, false),
     if
 	(MatchSpec == #vcard_search{_ = '_'}) and (not AllowReturnAll) ->
 	    [];
@@ -474,7 +495,8 @@ search(LServer, Data) ->
 		    ?ERROR_MSG("~p", [Reason]),
 		    [];
 		Rs ->
-		    case gen_mod:get_module_opt(?MODULE, matches, ?JUD_MATCHES) of
+		    case gen_mod:get_module_opt(LServer, ?MODULE,
+						matches, ?JUD_MATCHES) of
 			infinity ->
 			    Rs;
 			Val when is_integer(Val) and (Val > 0) ->
@@ -501,8 +523,8 @@ filter_fields([{SVar, [Val]} | Ds], Match, LServer)
     LVal = stringprep:tolower(Val),
     NewMatch = case SVar of
                    "user" ->
-		       case gen_mod:get_module_opt(
-			      ?MODULE, search_all_hosts, true) of
+		       case gen_mod:get_module_opt(LServer, ?MODULE,
+						   search_all_hosts, true) of
 			   true ->
 			       Match#vcard_search{luser = make_val(LVal)};
 			   false ->
