@@ -3,44 +3,46 @@
 %%% Author  : Alexey Shchepin <alexey@sevcom.net>
 %%% Purpose : ODBC connections supervisor
 %%% Created : 22 Dec 2004 by Alexey Shchepin <alexey@sevcom.net>
-%%% Id      : $Id: ejabberd_odbc_sup.erl 295 2004-12-29 23:10:14Z aleksey $
+%%% Id      : $Id: ejabberd_odbc_sup.erl 374 2005-07-13 03:24:13Z alexey $
 %%%----------------------------------------------------------------------
 
 -module(ejabberd_odbc_sup).
 -author('alexey@sevcom.net').
--vsn('$Revision$ ').
+-vsn('$Revision: 374 $ ').
 
--export([start_link/0,
+-export([start_link/1,
 	 init/1,
-	 get_pids/0,
-	 get_random_pid/0
+	 get_pids/1,
+	 get_random_pid/1
 	]).
 
 -include("ejabberd.hrl").
 
-start_link() ->
-    supervisor:start_link({local, ?MODULE}, ?MODULE, []).
+start_link(Host) ->
+    supervisor:start_link({local, gen_mod:get_module_proc(Host, ?MODULE)},
+			  ?MODULE, [Host]).
 
-init(_) ->
+init([Host]) ->
     % TODO
     N = 10,
     {ok, {{one_for_one, 10, 1},
 	  lists:map(
 	    fun(I) ->
 		    {I,
-		     {ejabberd_odbc, start_link, []},
+		     {ejabberd_odbc, start_link, [Host]},
 		     transient,
 		     brutal_kill,
 		     worker,
 		     [?MODULE]}
 	    end, lists:seq(1, N))}}.
 
-get_pids() ->
+get_pids(Host) ->
+    Proc = gen_mod:get_module_proc(Host, ?MODULE),
     [Child ||
-	{_Id, Child, _Type, _Modules} <- supervisor:which_children(?MODULE),
+	{_Id, Child, _Type, _Modules} <- supervisor:which_children(Proc),
 	Child /= undefined].
 
-get_random_pid() ->
-    Pids = get_pids(),
+get_random_pid(Host) ->
+    Pids = get_pids(Host),
     lists:nth(erlang:phash(now(), length(Pids)), Pids).
 
