@@ -3,12 +3,12 @@
 %%% Author  : Alexey Shchepin <alexey@sevcom.net>
 %%% Purpose : Interface to openssl
 %%% Created : 24 Jul 2004 by Alexey Shchepin <alexey@sevcom.net>
-%%% Id      : $Id: tls.erl 252 2004-08-01 20:12:20Z aleksey $
+%%% Id      : $Id: tls.erl 430 2005-11-03 05:04:54Z alexey $
 %%%----------------------------------------------------------------------
 
 -module(tls).
 -author('alexey@sevcom.net').
--vsn('$Revision$ ').
+-vsn('$Revision: 430 $ ').
 
 -behaviour(gen_server).
 
@@ -17,6 +17,8 @@
 	 send/2,
 	 recv/2, recv/3, recv_data/2,
 	 close/1,
+	 get_peer_certificate/1,
+	 get_verify_result/1,
 	 test/0]).
 
 %% Internal exports, call-back functions.
@@ -27,11 +29,14 @@
 	 code_change/3,
 	 terminate/2]).
 
--define(SET_CERTIFICATE_FILE, 1).
--define(SET_ENCRYPTED_INPUT,  2).
--define(SET_DECRYPTED_OUTPUT, 3).
--define(GET_ENCRYPTED_OUTPUT, 4).
--define(GET_DECRYPTED_INPUT,  5).
+-define(SET_CERTIFICATE_FILE_ACCEPT, 1).
+-define(SET_CERTIFICATE_FILE_CONNECT, 2).
+-define(SET_ENCRYPTED_INPUT,  3).
+-define(SET_DECRYPTED_OUTPUT, 4).
+-define(GET_ENCRYPTED_OUTPUT, 5).
+-define(GET_DECRYPTED_INPUT,  6).
+-define(GET_PEER_CERTIFICATE, 7).
+-define(GET_VERIFY_RESULT,    8).
 
 -record(tlssock, {tcpsock, tlsport}).
 
@@ -42,9 +47,12 @@ start_link() ->
     gen_server:start_link({local, ?MODULE}, ?MODULE, [], []).
 
 init([]) ->
-    ok = erl_ddll:load_driver(ejabberd:get_so_path(), tls_drv),
+    case erl_ddll:load_driver(ejabberd:get_so_path(), tls_drv) of
+	ok -> ok;
+	{error, already_loaded} -> ok
+    end,
     Port = open_port({spawn, tls_drv}, [binary]),
-    Res = port_control(Port, ?SET_CERTIFICATE_FILE, "./ssl.pem" ++ [0]),
+    Res = port_control(Port, ?SET_CERTIFICATE_FILE_ACCEPT, "./ssl.pem" ++ [0]),
     case Res of
 	<<0>> ->
 	    %ets:new(iconv_table, [set, public, named_table]),
@@ -65,15 +73,16 @@ handle_call(_, _, State) ->
 handle_cast(_, State) ->
     {noreply, State}.
 
-handle_info({'EXIT', Pid, Reason}, Port) ->
-    {noreply, Port};
-
 handle_info({'EXIT', Port, Reason}, Port) ->
     {stop, {port_died, Reason}, Port};
+
+handle_info({'EXIT', _Pid, _Reason}, Port) ->
+    {noreply, Port};
+
 handle_info(_, State) ->
     {noreply, State}.
 
-code_change(OldVsn, State, Extra) ->
+code_change(_OldVsn, State, _Extra) ->
     {ok, State}.
 
 terminate(_Reason, Port) ->
@@ -84,10 +93,18 @@ terminate(_Reason, Port) ->
 tcp_to_tls(TCPSocket, Options) ->
     case lists:keysearch(certfile, 1, Options) of
 	{value, {certfile, CertFile}} ->
-	    ok = erl_ddll:load_driver(ejabberd:get_so_path(), tls_drv),
+	    case erl_ddll:load_driver(ejabberd:get_so_path(), tls_drv) of
+		ok -> ok;
+		{error, already_loaded} -> ok
+	    end,
 	    Port = open_port({spawn, tls_drv}, [binary]),
-	    case port_control(Port, ?SET_CERTIFICATE_FILE,
-			      CertFile ++ [0]) of
+	    Command = case lists:member(connect, Options) of
+			  true ->
+			      ?SET_CERTIFICATE_FILE_CONNECT;
+			  false ->
+			      ?SET_CERTIFICATE_FILE_ACCEPT
+		      end,
+	    case port_control(Port, Command, CertFile ++ [0]) of
 		<<0>> ->
 		    {ok, #tlssock{tcpsock = TCPSocket, tlsport = Port}};
 		<<1, Error/binary>> ->
@@ -103,7 +120,7 @@ tls_to_tcp(#tlssock{tcpsock = TCPSocket, tlsport = Port}) ->
 
 recv(Socket, Length) ->
     recv(Socket, Length, infinity).
-recv(#tlssock{tcpsock = TCPSocket, tlsport = Port} = TLSSock,
+recv(#tlssock{tcpsock = TCPSocket} = TLSSock,
      Length, Timeout) ->
     case gen_tcp:recv(TCPSocket, Length, Timeout) of
 	{ok, Packet} ->
@@ -121,6 +138,7 @@ recv_data(#tlssock{tcpsock = TCPSocket, tlsport = Port}, Packet) ->
 			<<0, Out/binary>> ->
 			    case gen_tcp:send(TCPSocket, Out) of
 				ok ->
+				    %io:format("IN: ~p~n", [{TCPSocket, binary_to_list(In)}]),
 				    {ok, In};
 				Error ->
 				    Error
@@ -138,6 +156,7 @@ recv_data(#tlssock{tcpsock = TCPSocket, tlsport = Port}, Packet) ->
 send(#tlssock{tcpsock = TCPSocket, tlsport = Port}, Packet) ->
     case port_control(Port, ?SET_DECRYPTED_OUTPUT, Packet) of
 	<<0>> ->
+	    %io:format("OUT: ~p~n", [{TCPSocket, lists:flatten(Packet)}]),
 	    case port_control(Port, ?GET_ENCRYPTED_OUTPUT, []) of
 		<<0, Out/binary>> ->
 		    gen_tcp:send(TCPSocket, Out);
@@ -145,7 +164,14 @@ send(#tlssock{tcpsock = TCPSocket, tlsport = Port}, Packet) ->
 		    {error, binary_to_list(Error)}
 	    end;
 	<<1, Error/binary>> ->
-	    {error, binary_to_list(Error)}
+	    {error, binary_to_list(Error)};
+	<<2>> -> % Dirty hack
+	    receive
+		{timeout, _Timer, _} ->
+		    {error, timeout}
+	    after 100 ->
+		    send(#tlssock{tcpsock = TCPSocket, tlsport = Port}, Packet)
+	    end
     end.
 
 
@@ -153,12 +179,33 @@ close(#tlssock{tcpsock = TCPSocket, tlsport = Port}) ->
     gen_tcp:close(TCPSocket),
     port_close(Port).
 
+get_peer_certificate(#tlssock{tlsport = Port}) ->
+    case port_control(Port, ?GET_PEER_CERTIFICATE, []) of
+	<<0, BCert/binary>> ->
+	    case catch ssl_pkix:decode_cert(BCert, [pkix]) of
+		{ok, Cert} ->
+		    {ok, Cert};
+		_ ->
+		    error
+	    end;
+	<<1>> ->
+	    error
+    end.
+
+get_verify_result(#tlssock{tlsport = Port}) ->
+    <<Res>> = port_control(Port, ?GET_VERIFY_RESULT, []),
+    Res.
+
 
 test() ->
-    ok = erl_ddll:load_driver(ejabberd:get_so_path(), tls_drv),
+    case erl_ddll:load_driver(ejabberd:get_so_path(), tls_drv) of
+	ok -> ok;
+	{error, already_loaded} -> ok
+    end,
     Port = open_port({spawn, tls_drv}, [binary]),
     io:format("open_port: ~p~n", [Port]),
-    PCRes = port_control(Port, ?SET_CERTIFICATE_FILE, "./ssl.pem" ++ [0]),
+    PCRes = port_control(Port, ?SET_CERTIFICATE_FILE_ACCEPT,
+			 "./ssl.pem" ++ [0]),
     io:format("port_control: ~p~n", [PCRes]),
     {ok, ListenSocket} = gen_tcp:listen(1234, [binary,
 					       {packet, 0}, 
