@@ -1,9 +1,10 @@
-/* $Id: tls_drv.c 327 2005-04-24 12:09:47Z alexey $ */
+/* $Id: tls_drv.c 430 2005-11-03 05:04:54Z alexey $ */
 
 #include <stdio.h>
 #include <string.h>
 #include <erl_driver.h>
 #include <openssl/ssl.h>
+#include <openssl/err.h>
 
 
 #define BUF_SIZE 1024
@@ -45,11 +46,19 @@ static void tls_drv_stop(ErlDrvData handle)
 }
 
 
-#define SET_CERTIFICATE_FILE 1
-#define SET_ENCRYPTED_INPUT  2
-#define SET_DECRYPTED_OUTPUT 3
-#define GET_ENCRYPTED_OUTPUT 4
-#define GET_DECRYPTED_INPUT  5
+static int verify_callback(int preverify_ok, X509_STORE_CTX *ctx)
+{
+   return 1;
+}
+
+#define SET_CERTIFICATE_FILE_ACCEPT 1
+#define SET_CERTIFICATE_FILE_CONNECT 2
+#define SET_ENCRYPTED_INPUT  3
+#define SET_DECRYPTED_OUTPUT 4
+#define GET_ENCRYPTED_OUTPUT 5
+#define GET_DECRYPTED_INPUT  6
+#define GET_PEER_CERTIFICATE 7
+#define GET_VERIFY_RESULT    8
 
 
 #define die_unless(cond, errstr)				\
@@ -73,11 +82,13 @@ static int tls_drv_control(ErlDrvData handle,
    int res;
    int size;
    ErlDrvBinary *b;
+   X509 *cert;
 
    switch (command)
    {
-      case SET_CERTIFICATE_FILE:
-	 d->ctx = SSL_CTX_new(SSLv23_server_method());
+      case SET_CERTIFICATE_FILE_ACCEPT:
+      case SET_CERTIFICATE_FILE_CONNECT:
+	 d->ctx = SSL_CTX_new(SSLv23_method());
 	 die_unless(d->ctx, "SSL_CTX_new failed");
 
 	 res = SSL_CTX_use_certificate_file(d->ctx, buf, SSL_FILETYPE_PEM);
@@ -89,6 +100,15 @@ static int tls_drv_control(ErlDrvData handle,
 	 res = SSL_CTX_check_private_key(d->ctx);
 	 die_unless(res > 0, "SSL_CTX_check_private_key failed");
 
+	 SSL_CTX_set_default_verify_paths(d->ctx);
+
+	 if (command == SET_CERTIFICATE_FILE_ACCEPT)
+	 {
+	    SSL_CTX_set_verify(d->ctx,
+			       SSL_VERIFY_PEER|SSL_VERIFY_CLIENT_ONCE,
+			       verify_callback);
+	 }
+	 
 	 d->ssl = SSL_new(d->ctx);
 	 die_unless(d->ssl, "SSL_new failed");
 
@@ -97,7 +117,10 @@ static int tls_drv_control(ErlDrvData handle,
 
 	 SSL_set_bio(d->ssl, d->bio_read, d->bio_write);
 
-	 SSL_set_accept_state(d->ssl);
+	 if (command == SET_CERTIFICATE_FILE_ACCEPT)
+	    SSL_set_accept_state(d->ssl);
+	 else
+	    SSL_set_connect_state(d->ssl);
 	 break;
       case SET_ENCRYPTED_INPUT:
 	 die_unless(d->ssl, "SSL not initialized");
@@ -106,6 +129,19 @@ static int tls_drv_control(ErlDrvData handle,
       case SET_DECRYPTED_OUTPUT:
 	 die_unless(d->ssl, "SSL not initialized");
 	 res = SSL_write(d->ssl, buf, len);
+	 if (res <= 0) 
+	 {
+	    res = SSL_get_error(d->ssl, res);
+	    if (res == SSL_ERROR_WANT_READ || res == SSL_ERROR_WANT_WRITE) 
+	    {
+	       b = driver_alloc_binary(1);
+	       b->orig_bytes[0] = 2;
+	       *rbuf = (char *)b;
+	       return 1;
+	    } else {
+	       die_unless(0, "SSL_write failed");
+	    }
+	 }
 	 break;
       case GET_ENCRYPTED_OUTPUT:
 	 die_unless(d->ssl, "SSL not initialized");
@@ -128,13 +164,10 @@ static int tls_drv_control(ErlDrvData handle,
       case GET_DECRYPTED_INPUT:
 	 if (!SSL_is_init_finished(d->ssl))
 	 {
-	    //printf("Doing SSL_accept\r\n");
-	    res = SSL_accept(d->ssl);
-	    //if (res == 0)
-	    //   printf("SSL_accept returned zero\r\n");
-	    if (res < 0)
+	    res = SSL_do_handshake(d->ssl);
+	    if (res <= 0)
 	       die_unless(SSL_get_error(d->ssl, res) == SSL_ERROR_WANT_READ,
-			  "SSL_accept failed");
+			  "SSL_do_handshake failed");
 	 } else {
 	    size = BUF_SIZE + 1;
 	    rlen = 1;
@@ -166,6 +199,37 @@ static int tls_drv_control(ErlDrvData handle,
 	    return rlen;
 	 }
 	 break;
+      case GET_PEER_CERTIFICATE:
+	 cert = SSL_get_peer_certificate(d->ssl);
+	 if (cert == NULL)
+	 {
+	    b = driver_alloc_binary(1);
+	    b->orig_bytes[0] = 1;
+	    *rbuf = (char *)b;
+	    return 1;
+	 } else {
+	    unsigned char *tmp_buf;
+	    rlen = i2d_X509(cert, NULL);
+	    if (rlen >= 0)
+	    {
+	       rlen++;
+	       b = driver_alloc_binary(rlen);
+	       b->orig_bytes[0] = 0;
+	       tmp_buf = &b->orig_bytes[1];
+	       i2d_X509(cert, &tmp_buf);
+	       X509_free(cert);
+	       *rbuf = (char *)b;
+	       return rlen;
+	    } else
+	       X509_free(cert);
+	 }
+	 break;
+      case GET_VERIFY_RESULT:
+	 b = driver_alloc_binary(1);
+	 b->orig_bytes[0] = SSL_get_verify_result(d->ssl);
+	 *rbuf = (char *)b;
+	 return 1;
+	 break;
    }
 
    b = driver_alloc_binary(1);
