@@ -3,17 +3,18 @@
 %%% Author  : Alexey Shchepin <alexey@sevcom.net>
 %%% Purpose : 
 %%% Created : 27 Feb 2004 by Alexey Shchepin <alexey@sevcom.net>
-%%% Id      : $Id: ejabberd_http.erl 353 2005-05-22 23:29:54Z alexey $
+%%% Id      : $Id: ejabberd_http.erl 444 2005-11-26 18:56:39Z alexey $
 %%%----------------------------------------------------------------------
 
 -module(ejabberd_http).
 -author('alexey@sevcom.net').
--vsn('$Revision: 353 $ ').
+-vsn('$Revision: 444 $ ').
 
 %% External exports
 -export([start/2,
 	 start_link/2,
-	 receive_headers/1]).
+	 receive_headers/1,
+	 url_encode/1]).
 
 -include("ejabberd.hrl").
 -include("jlib.hrl").
@@ -41,7 +42,8 @@
 	"\"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">\n").
 
 -define(HTML_DOCTYPE,
-	"<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">\n").
+	"<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\" "
+	"\"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">\n").
 
 
 start(SockData, Opts) ->
@@ -652,6 +654,46 @@ parse_urlencoded(undefined, _, _, _) ->
     [].
 
 
+url_encode([H|T]) ->
+    if
+	H >= $a, $z >= H ->
+	    [H|url_encode(T)];
+	H >= $A, $Z >= H ->
+	    [H|url_encode(T)];
+	H >= $0, $9 >= H ->
+	    [H|url_encode(T)];
+	H == $_; H == $.; H == $-; H == $/; H == $: -> % FIXME: more..
+	    [H|url_encode(T)];
+	true ->
+	    case integer_to_hex(H) of
+		[X, Y] ->
+		    [$%, X, Y | url_encode(T)];
+		[X] ->
+		    [$%, $0, X | url_encode(T)]
+	    end
+    end;
+
+url_encode([]) ->
+    [].
+
+integer_to_hex(I) ->
+    case catch erlang:integer_to_list(I, 16) of
+	{'EXIT', _} ->
+	    old_integer_to_hex(I);
+	Int ->
+	    Int
+    end.
+
+
+old_integer_to_hex(I) when I<10 ->
+    integer_to_list(I);
+old_integer_to_hex(I) when I<16 ->
+    [I-10+$A];
+old_integer_to_hex(I) when I>=16 ->
+    N = trunc(I/16),
+    old_integer_to_hex(N) ++ old_integer_to_hex(I rem 16).
+
+
 % The following code is mostly taken from yaws_ssl.erl
 
 parse_request(State, Data) ->
