@@ -3,12 +3,12 @@
 %%% Author  : Alexey Shchepin <alexey@sevcom.net>
 %%% Purpose : 
 %%% Created : 28 Feb 2004 by Alexey Shchepin <alexey@sevcom.net>
-%%% Id      : $Id: ejabberd_web.erl 307 2005-04-17 18:08:34Z tmallard $
+%%% Id      : $Id: ejabberd_web.erl 370 2005-06-20 03:18:13Z alexey $
 %%%----------------------------------------------------------------------
 
 -module(ejabberd_web).
 -author('alexey@sevcom.net').
--vsn('$Revision$ ').
+-vsn('$Revision: 370 $ ').
 
 %% External exports
 -export([make_xhtml/1,
@@ -49,6 +49,34 @@ make_xhtml(Els) ->
 		      {"value", Value}])).
 
 
+process_get({_, true},
+	    #request{us = US,
+		     path = ["admin", "server", SHost | RPath],
+		     q = Query,
+		     lang = Lang} = Request) ->
+    Host = jlib:nameprep(SHost),
+    case lists:member(Host, ?MYHOSTS) of
+	true ->
+	    case US of
+		{User, Server} ->
+		    case acl:match_rule(
+			   Host, configure, jlib:make_jid(User, Server, "")) of
+			deny ->
+			    {401, [], make_xhtml([?XC("h1", "Not Allowed")])};
+			allow ->
+			    ejabberd_web_admin:process_admin(
+			      Host, Request#request{path = RPath})
+		    end;
+		undefined ->
+		    {401,
+		     [{"WWW-Authenticate", "basic realm=\"ejabberd\""}],
+		     ejabberd_web:make_xhtml([{xmlelement, "h1", [],
+					       [{xmlcdata, "401 Unauthorized"}]}])}
+	    end;
+	false ->
+	    {404, [], make_xhtml([?XC("h1", "Not found")])}
+    end;
+
 process_get({_, true},
 	    #request{us = US,
 		     path = ["admin" | RPath],
@@ -56,12 +84,13 @@ process_get({_, true},
 		     lang = Lang} = Request) ->
     case US of
 	{User, Server} ->
-	    case acl:match_rule(configure, jlib:make_jid(User, Server, "")) of
+	    case acl:match_rule(
+		   global, configure, jlib:make_jid(User, Server, "")) of
 		deny ->
 		    {401, [], make_xhtml([?XC("h1", "Not Allowed")])};
 		allow ->
 		    ejabberd_web_admin:process_admin(
-		      Request#request{path = RPath})
+		      global, Request#request{path = RPath})
 	    end;
 	undefined ->
 	    {401,
